@@ -3,12 +3,10 @@ fn main() {
     // Create a client object
     let mut client = uflow::Client::bind_any_ipv4().unwrap();
 
-    // The client will send data on only one transmission channel
-    let cfg = uflow::EndpointConfig::default()
-        .channel_count(1);
-
     // Initiate the connection to the server
-    let mut server_peer = client.connect("127.0.0.1:8888", cfg).expect("Invalid address");
+    let address = "127.0.0.1:8888";
+    let config = uflow::EndpointConfig::default();
+    let mut server_peer = client.connect(address, config).expect("Invalid address");
 
     let mut send_counter = 0;
     let mut message_counter = 0;
@@ -42,7 +40,7 @@ fn main() {
         if send_counter == 10 {
             let packet_data: Box<[u8]> = format!("Hello world {}!", message_counter).as_bytes().into();
 
-            server_peer.send(packet_data, 0, uflow::SendMode::Reliable);
+            server_peer.send(packet_data, 0, uflow::SendMode::Unreliable);
 
             send_counter = 0;
             message_counter += 1;
