@@ -18,9 +18,12 @@ pub struct Client {
 }
 
 impl Client {
-    /// Opens a UDP socket and creates a corresponding [`Client`](Self) object. The UDP socket is
-    /// bound to the provided address, and configured to be non-blocking. Any errors resulting from
-    /// socket initialization are forwarded to the caller.
+    /// Opens a non-blocking UDP socket bound to the provided address, and creates a corresponding
+    /// [`Client`](Self) object.
+    ///
+    /// # Error Handling
+    ///
+    /// Any errors resulting from socket initialization are forwarded to the caller.
     pub fn bind<A: net::ToSocketAddrs>(addr: A) -> Result<Self, std::io::Error> {
         let socket = net::UdpSocket::bind(addr)?;
 
@@ -45,8 +48,12 @@ impl Client {
     }
 
     /// Initiates a connection to a remote host at the specified address, and returns a
-    /// [`Peer`](peer::Peer) object representing the connection. Any errors resulting from address
-    /// resolution are forwarded to the caller.
+    /// [`Peer`](peer::Peer) object representing the connection.
+    ///
+    /// # Error Handling
+    ///
+    /// Any errors resulting from address resolution are forwarded to the caller. This function
+    /// will panic if the given endpoint configuration is not valid.
     pub fn connect<A: net::ToSocketAddrs>(&mut self, addr: A, cfg: endpoint::Config) -> Result<peer::Peer, std::io::Error> {
         assert!(cfg.is_valid(), "invalid endpoint config");
 
@@ -59,13 +66,18 @@ impl Client {
         return Ok(peer::Peer::new(address, endpoint_ref));
     }
 
-    /// Processes UDP frames received since the last call to [`step()`](Self::step), and
-    /// sends any pending outbound frames (acknowledgements, keep-alives, packet data, etc.).
+    /// Flushes pending outbound frames, reads as many UDP frames as possible from the internal
+    /// socket, and updates the states of active connections. Call
+    /// [`Peer::poll_events()`](peer::Peer::poll_events) after calling this function to retrieve
+    /// incoming packets and connection status updates for an individual peer.
     ///
-    /// Current, non-zombie [`Peer`](peer::Peer) objects will be updated as relevant data is
-    /// received. Call [`Peer::poll_events()`](peer::Peer::poll_events) after calling this method
-    /// to retrieve incoming packets and connection status updates for an individual peer.
+    /// *Note*: Internally, `uflow` uses the [leaky bucket
+    /// algorithm](https://en.wikipedia.org/wiki/Leaky_bucket) to control the rate at which UDP
+    /// frames are sent. To ensure that data is transferred smoothly, this function should be
+    /// called relatively frequently (a minimum of once per connection round-trip time).
     pub fn step(&mut self) {
+        self.flush();
+
         let mut frame_data_buf = [0; MAX_FRAME_SIZE];
 
         while let Ok((frame_size, address)) = self.socket.recv_from(&mut frame_data_buf) {
@@ -76,16 +88,18 @@ impl Client {
             }
         }
 
-        self.flush();
+        for (_, endpoint) in self.endpoints.iter_mut() {
+            endpoint.borrow_mut().step();
+        }
 
         self.endpoints.retain(|_, endpoint| !endpoint.borrow().is_zombie());
     }
 
-    /// Sends any pending outbound frames (acknowledgements, keep-alives, packet data, etc.).
+    /// Sends as many pending outbound frames (packet data, acknowledgements, keep-alives, etc.) as
+    /// possible for each peer.
     pub fn flush(&mut self) {
         for (&address, endpoint) in self.endpoints.iter_mut() {
             let ref mut data_sink = UdpFrameSink::new(&self.socket, address);
-
             endpoint.borrow_mut().flush(data_sink);
         }
     }
@@ -98,7 +112,6 @@ impl Client {
     fn handle_frame(&mut self, address: net::SocketAddr, frame: frame::Frame) {
         if let Some(endpoint) = self.endpoints.get_mut(&address) {
             let ref mut data_sink = UdpFrameSink::new(&self.socket, address);
-
             endpoint.borrow_mut().handle_frame(frame, data_sink);
         }
     }
