@@ -1,11 +1,9 @@
 
-use crate::SendMode;
-use crate::MAX_FRAME_SIZE;
-use crate::MAX_FRAME_WINDOW_SIZE;
-use crate::frame;
-
 use super::FrameSink;
 
+use crate::SendMode;
+use crate::frame;
+
 use std::time;
 
 mod emit;
@@ -27,11 +25,33 @@ mod packet_tests;
 const INITIAL_RTT_ESTIMATE_MS: u64 = 150;
 const INITIAL_RTO_ESTIMATE_MS: u64 = 4*INITIAL_RTT_ESTIMATE_MS;
 const MIN_SYNC_TIMEOUT_MS: u64 = 2000;
+const MIN_SYNC_KEEPALIVE_TIMEOUT_MS: u64 = 5000;
 
 pub trait PacketSink {
     fn send(&mut self, packet_data: Box<[u8]>);
 }
 
+pub struct Config {
+    pub tx_frame_base_id: u32,
+    pub rx_frame_base_id: u32,
+
+    pub tx_frame_window_size: u32,
+    pub rx_frame_window_size: u32,
+
+    pub tx_packet_base_id: u32,
+    pub rx_packet_base_id: u32,
+
+    pub tx_packet_window_size: u32,
+    pub rx_packet_window_size: u32,
+
+    pub tx_bandwidth_limit: u32,
+
+    pub tx_alloc_limit: usize,
+    pub rx_alloc_limit: usize,
+
+    pub keepalive: bool,
+}
+
 pub struct DatenMeister {
     packet_sender: packet_sender::PacketSender,
     pending_queue: pending_queue::PendingQueue,
@@ -43,40 +63,47 @@ pub struct DatenMeister {
 
     send_rate_comp: send_rate::SendRateComp,
 
+    now_ms: u64,
+    rtt_ms: u64,
+    rto_ms: u64,
+
     time_base: time::Instant,
     time_last_flushed: Option<time::Instant>,
     sync_timeout_base_ms: u64,
 
-    flush_alloc: usize,
+    flush_alloc: isize,
     flush_id: u32,
 
     sync_reply: bool,
+    sync_keepalive: bool,
 }
 
 impl DatenMeister {
-    pub fn new(tx_channels: usize, rx_channels: usize,
-               tx_alloc_limit: usize, rx_alloc_limit: usize,
-               tx_base_id: u32, rx_base_id: u32,
-               tx_bandwidth_limit: u32) -> Self {
+    pub fn new(config: Config) -> Self {
         Self {
-            packet_sender: packet_sender::PacketSender::new(tx_channels, tx_alloc_limit, tx_base_id),
+            packet_sender: packet_sender::PacketSender::new(config.tx_packet_window_size, config.tx_packet_base_id, config.tx_alloc_limit),
             pending_queue: pending_queue::PendingQueue::new(),
             resend_queue: resend_queue::ResendQueue::new(),
-            frame_queue: frame_queue::FrameQueue::new(tx_base_id, MAX_FRAME_WINDOW_SIZE, MAX_FRAME_WINDOW_SIZE),
+            frame_queue: frame_queue::FrameQueue::new(config.tx_frame_window_size, config.tx_frame_window_size, config.tx_frame_base_id),
 
-            packet_receiver: packet_receiver::PacketReceiver::new(rx_channels, rx_alloc_limit, rx_base_id),
-            frame_ack_queue: frame_ack_queue::FrameAckQueue::new(rx_base_id, MAX_FRAME_WINDOW_SIZE),
+            packet_receiver: packet_receiver::PacketReceiver::new(config.rx_packet_window_size, config.rx_packet_base_id, config.rx_alloc_limit),
+            frame_ack_queue: frame_ack_queue::FrameAckQueue::new(config.rx_frame_window_size, config.rx_frame_base_id),
 
-            send_rate_comp: send_rate::SendRateComp::new(tx_bandwidth_limit),
+            send_rate_comp: send_rate::SendRateComp::new(config.tx_bandwidth_limit),
+
+            now_ms: 0,
+            rtt_ms: 0,
+            rto_ms: 0,
 
             time_base: time::Instant::now(),
             time_last_flushed: None,
             sync_timeout_base_ms: 0,
 
-            flush_alloc: MAX_FRAME_SIZE,
+            flush_alloc: 0,
             flush_id: 0,
 
             sync_reply: false,
+            sync_keepalive: config.keepalive,
         }
     }
 
@@ -84,6 +111,10 @@ impl DatenMeister {
         self.send_rate_comp.rtt_s()
     }
 
+    pub fn send_buffer_size(&self) -> usize {
+        self.packet_sender.total_size()
+    }
+
     pub fn is_send_pending(&self) -> bool {
         self.packet_sender.pending_count() != 0 || self.pending_queue.len() != 0 || self.resend_queue.len() != 0
     }
@@ -129,16 +160,27 @@ impl DatenMeister {
         self.packet_sender.acknowledge(frame.packet_window_base_id);
     }
 
-    pub fn flush(&mut self, sink: &mut impl FrameSink) {
+    pub fn step(&mut self) {
         let now = time::Instant::now();
-        let now_ms = (now - self.time_base).as_millis() as u64;
 
+        let now_ms = (now - self.time_base).as_millis() as u64;
         let rtt_ms = self.send_rate_comp.rtt_ms().unwrap_or(INITIAL_RTT_ESTIMATE_MS);
         let rto_ms = self.send_rate_comp.rto_ms().unwrap_or(INITIAL_RTO_ESTIMATE_MS);
 
+        // Store these values for subsequent flush()
+        self.now_ms = now_ms;
+        self.rtt_ms = rtt_ms;
+        self.rto_ms = rto_ms;
+
         // Forget old frame data
         self.frame_queue.forget_frames(now_ms.saturating_sub(rtt_ms*4), self.send_rate_comp.rtt_ms());
 
+        // Fill flush allocation
+        self.fill_flush_alloc(now);
+
+        // Ignore previous TimeSensitive packets
+        self.flush_id = self.flush_id.wrapping_add(1);
+
         // Update send rate value
         let ref mut frame_queue = self.frame_queue;
         self.send_rate_comp.step(now_ms, frame_queue.get_feedback(now_ms),
@@ -146,12 +188,11 @@ impl DatenMeister {
                 frame_queue.reset_loss_rate(new_loss_rate);
             }
         );
+    }
 
-        // Fill flush allocation according to send rate
-        self.fill_flush_alloc(now);
-
+    pub fn flush(&mut self, sink: &mut impl FrameSink) {
         // Send as many frames as possible
-        self.emit_frames(now_ms, rtt_ms, rto_ms, sink);
+        self.emit_frames(self.now_ms, self.rtt_ms, self.rto_ms, self.flush_id, sink);
     }
 
     fn fill_flush_alloc(&mut self, now: time::Instant) {
@@ -160,23 +201,15 @@ impl DatenMeister {
             let rtt_s = self.send_rate_comp.rtt_s();
 
             let delta_time = (now - time_last_flushed).as_secs_f64();
-            let new_bytes = (send_rate * delta_time).round() as usize;
-            let alloc_max = ((send_rate * rtt_s.unwrap_or(0.0)).round() as usize).max(MAX_FRAME_SIZE);
+            let new_bytes = (send_rate * delta_time).round() as isize;
+            let alloc_max = (send_rate * rtt_s.unwrap_or(0.0)).round() as isize;
 
             self.flush_alloc = self.flush_alloc.saturating_add(new_bytes).min(alloc_max);
         }
         self.time_last_flushed = Some(now);
     }
 
-    fn emit_frames(&mut self, now_ms: u64, rtt_ms: u64, rto_ms: u64, sink: &mut impl FrameSink) {
-        let flush_id = self.flush_id;
-        self.flush_id = self.flush_id.wrapping_add(1);
-
-        match self.emit_sync_frame(now_ms, rto_ms, sink) {
-            Err(_) => return,
-            Ok(_) => (),
-        }
-
+    fn emit_frames(&mut self, now_ms: u64, rtt_ms: u64, rto_ms: u64, flush_id: u32, sink: &mut impl FrameSink) {
         match self.emit_ack_frames(sink) {
             Err(_) => return,
             Ok(_) => (),
@@ -186,12 +219,18 @@ impl DatenMeister {
             Err(_) => return,
             Ok(_) => (),
         }
+
+        match self.emit_sync_frame(now_ms, rto_ms, sink) {
+            Err(_) => return,
+            Ok(_) => (),
+        }
     }
 
     fn emit_sync_frame(&mut self, now_ms: u64, rto_ms: u64, sink: &mut impl FrameSink) -> Result<(),()> {
+        let elapsed_ms = now_ms - self.sync_timeout_base_ms;
         let sync_timeout_ms = rto_ms.max(MIN_SYNC_TIMEOUT_MS);
 
-        if now_ms - self.sync_timeout_base_ms >= sync_timeout_ms {
+        if elapsed_ms >= sync_timeout_ms {
             let next_frame_id =
                 if self.frame_queue.next_id() != self.frame_queue.base_id() {
                     Some(self.frame_queue.next_id())
@@ -207,19 +246,27 @@ impl DatenMeister {
                     None
                 };
 
-            // TODO: Only send a (None, None) sync frame if application requests keepalives
+            if next_frame_id.is_none() && next_packet_id.is_none() {
+                if self.sync_keepalive {
+                    if elapsed_ms < MIN_SYNC_KEEPALIVE_TIMEOUT_MS {
+                        return Ok(());
+                    }
+                } else {
+                    return Ok(());
+                }
+            }
 
-            if frame::serial::SYNC_FRAME_SIZE > self.flush_alloc {
+            if self.flush_alloc < 0 {
                 return Err(());
             }
 
             let frame = frame::Frame::SyncFrame(frame::SyncFrame { next_frame_id, next_packet_id });
 
             use frame::serial::Serialize;
-            let frame_data = frame.write();
+            let frame_bytes = frame.write();
 
-            sink.send(&frame_data);
-            self.flush_alloc -= frame_data.len();
+            sink.send(&frame_bytes);
+            self.flush_alloc -= frame_bytes.len() as isize;
             self.sync_timeout_base_ms = now_ms;
         }
 
@@ -238,11 +285,18 @@ impl DatenMeister {
 
         let emit_cb = |frame_bytes: Box<[u8]>| {
             sink.send(&frame_bytes);
-            *flush_alloc -= frame_bytes.len();
+            *flush_alloc -= frame_bytes.len() as isize;
             *sync_reply = false;
         };
 
-        let mut afe = emit::AckFrameEmitter::new(frame_window_base_id, packet_window_base_id, sync_reply_init, flush_alloc_init, emit_cb);
+        let mut afe = emit::AckFrameEmitter::new(frame_window_base_id, packet_window_base_id, flush_alloc_init, emit_cb);
+
+        if sync_reply_init {
+            match afe.push_dud() {
+                Err(_) => return Err(()),
+                Ok(_) => (),
+            }
+        }
 
         while let Some(ack_group) = self.frame_ack_queue.peek() {
             match afe.push(ack_group) {
@@ -268,7 +322,7 @@ impl DatenMeister {
         let emit_cb = |frame_bytes: Box<[u8]>| {
             sink.send(&frame_bytes);
             send_rate_comp.notify_frame_sent(now_ms);
-            *flush_alloc -= frame_bytes.len();
+            *flush_alloc -= frame_bytes.len() as isize;
             *sync_timeout_base_ms = now_ms;
         };
 
@@ -288,7 +342,9 @@ impl DatenMeister {
                 }
 
                 match dfe.push(&packet_rc, entry.fragment_ref.fragment_id, true) {
-                    Err(_) => return Err(()),
+                    // Being window-limited does not preclude further sends
+                    Err(emit::DataPushError::WindowLimited) => return Ok(()),
+                    Err(emit::DataPushError::SizeLimited) => return Err(()),
                     Ok(_) => (),
                 }
 
@@ -332,7 +388,9 @@ impl DatenMeister {
                     }
 
                     match dfe.push(&packet_rc, entry.fragment_ref.fragment_id, entry.resend) {
-                        Err(_) => return Err(()),
+                        // Being window-limited does not preclude further sends
+                        Err(emit::DataPushError::WindowLimited) => return Ok(()),
+                        Err(emit::DataPushError::SizeLimited) => return Err(()),
                         Ok(_) => (),
                     }
 
@@ -360,9 +418,11 @@ mod tests {
 
     use crate::SendMode;
     use crate::frame::Datagram;
-    use crate::frame::FragmentId;
 
     use crate::MAX_FRAGMENT_SIZE;
+    use crate::MAX_FRAME_SIZE;
+    use crate::MAX_FRAME_WINDOW_SIZE;
+    use crate::MAX_PACKET_WINDOW_SIZE;
 
     use std::collections::VecDeque;
 
@@ -386,17 +446,40 @@ mod tests {
 
     struct TestApparatus {
         dm: DatenMeister,
+        flush_id: u32,
     }
 
     impl TestApparatus {
         fn new() -> Self {
+            let config = Config {
+                tx_frame_window_size: MAX_FRAME_WINDOW_SIZE,
+                rx_frame_window_size: MAX_FRAME_WINDOW_SIZE,
+
+                tx_frame_base_id: 0,
+                rx_frame_base_id: 0,
+
+                tx_packet_window_size: MAX_PACKET_WINDOW_SIZE,
+                rx_packet_window_size: MAX_PACKET_WINDOW_SIZE,
+
+                tx_packet_base_id: 0,
+                rx_packet_base_id: 0,
+
+                tx_bandwidth_limit: 100_000,
+
+                tx_alloc_limit: MAX_FRAGMENT_SIZE * MAX_PACKET_WINDOW_SIZE as usize,
+                rx_alloc_limit: MAX_FRAGMENT_SIZE * MAX_PACKET_WINDOW_SIZE as usize,
+
+                keepalive: true,
+            };
+
             Self {
-                dm: DatenMeister::new(1, 1, 10_000, 10_000, 0, 0, 100_000)
+                dm: DatenMeister::new(config),
+                flush_id: 0,
             }
         }
 
         fn set_flush_id(&mut self, flush_id: u32) {
-            self.dm.flush_id = flush_id;
+            self.flush_id = flush_id;
         }
 
         fn receive_sync(&mut self, frame: frame::SyncFrame) {
@@ -419,12 +502,12 @@ mod tests {
             self.dm.frame_queue.acknowledge_group(group, rtt_ms);
         }
 
-        fn emit_frames(&mut self, now_ms: u64, rtt_ms: u64, max_send_size: usize) -> VecDeque<Box<[u8]>> {
+        fn emit_frames(&mut self, now_ms: u64, rtt_ms: u64, flush_alloc: isize) -> VecDeque<Box<[u8]>> {
             let mut test_sink = TestSink::new();
 
-            self.dm.flush_alloc = max_send_size;
+            self.dm.flush_alloc = flush_alloc;
 
-            self.dm.emit_frames(now_ms, rtt_ms, 4*rtt_ms, &mut test_sink);
+            self.dm.emit_frames(now_ms, rtt_ms, 4*rtt_ms, self.flush_id, &mut test_sink);
 
             return test_sink.emitted;
         }
@@ -492,7 +575,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id: 0,
+            fragment_id_last: 0,
             data: vec![ 0, 0, 0 ].into_boxed_slice(),
         };
 
@@ -517,7 +601,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 1 },
+            fragment_id: 0,
+            fragment_id_last: 1,
             data: packet_data[ .. MAX_FRAGMENT_SIZE].into(),
         };
 
@@ -526,7 +611,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 1, last: 1 },
+            fragment_id: 1,
+            fragment_id_last: 1,
             data: packet_data[MAX_FRAGMENT_SIZE .. ].into(),
         };
 
@@ -537,36 +623,6 @@ mod tests {
         assert_eq!(frames[1].len(), MAX_FRAME_SIZE);
     }
 
-    // Packets should be resent [1, 2, 4, 4, ... 4] RTTs after the previous send.
-    #[test]
-    fn resend_timing() {
-        let rtt_ms = 100;
-
-        let mut ta = TestApparatus::new();
-
-        let p0 = (0 .. 400).map(|i| i as u8).collect::<Vec<u8>>().into_boxed_slice();
-        ta.enqueue_packet(p0.clone(), 0, SendMode::Persistent);
-
-        let frames = ta.emit_frames(0, rtt_ms, MAX_FRAME_SIZE);
-        assert_eq!(frames.len(), 1);
-
-        let frames = ta.emit_frames(1, rtt_ms, MAX_FRAME_SIZE);
-        assert_eq!(frames.len(), 0);
-
-        let resend_times = [ rtt_ms, 3*rtt_ms, 7*rtt_ms, 11*rtt_ms, 15*rtt_ms, 19*rtt_ms, 23*rtt_ms ];
-
-        for &time_ms in resend_times.iter() {
-            let frames = ta.emit_frames(time_ms - 1, rtt_ms, MAX_FRAME_SIZE);
-            assert_eq!(frames.len(), 0);
-
-            let frames = ta.emit_frames(time_ms    , rtt_ms, MAX_FRAME_SIZE);
-            assert_eq!(frames.len(), 1);
-
-            let frames = ta.emit_frames(time_ms + 1, rtt_ms, MAX_FRAME_SIZE);
-            assert_eq!(frames.len(), 0);
-        }
-    }
-
     // Time sensitive packet IDs should not be resent if the flush ID does not match.
     #[test]
     fn time_sensitive_drop() {
@@ -588,7 +644,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id: 0,
+            fragment_id_last: 0,
             data: vec![ 1, 1, 1 ].into_boxed_slice(),
         };
 
@@ -629,7 +686,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id: 0,
+            fragment_id_last: 0,
             data: p4,
         };
 
@@ -674,7 +732,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id: 0,
+            fragment_id_last: 0,
             data: p1,
         };
 
@@ -684,10 +743,10 @@ mod tests {
     // Simple sync case for which both the frame and packet windows are resynchronized.
     #[test]
     fn sync_frame_and_packet_window() {
-        let now_ms = 0;
-        let rtt_ms = 100;
+        let rtt_ms = INITIAL_RTT_ESTIMATE_MS;
 
         let mut ta = TestApparatus::new();
+        let mut now_ms = 0;
 
         for _ in 0 .. 5 {
             ta.enqueue_packet(vec![ 0; MAX_FRAGMENT_SIZE ].into_boxed_slice(), 0, SendMode::Unreliable);
@@ -696,47 +755,46 @@ mod tests {
         let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
         assert_eq!(frames.len(), 5);
 
-        let frames = ta.emit_frames(now_ms + MIN_SYNC_TIMEOUT_MS, rtt_ms, 10000);
+        now_ms += MIN_SYNC_TIMEOUT_MS;
+
+        let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
         assert_eq!(frames.len(), 1);
 
         test_sync_frame(&frames[0], frame::SyncFrame { next_frame_id: Some(5), next_packet_id: Some(5) });
     }
 
-    // Sync case for which packets exist in the resend/pending queues, so only the frame window is
-    // resynchronized.
+    // Sync case for which packets exist in the resend/pending queues, and the frame transfer
+    // window is full, so only the frame window is resynchronized.
     #[test]
     fn sync_frame_window_only() {
-        let now_ms = 0;
-        let rtt_ms = 100;
+        let rtt_ms = INITIAL_RTT_ESTIMATE_MS;
 
         let mut ta = TestApparatus::new();
+        let mut now_ms = 0;
 
-        for _ in 0 .. 5 {
+        for _ in 0 .. MAX_FRAME_WINDOW_SIZE {
             ta.enqueue_packet(vec![ 0; MAX_FRAGMENT_SIZE ].into_boxed_slice(), 0, SendMode::Persistent);
         }
 
+        let frames = ta.emit_frames(now_ms, rtt_ms, MAX_FRAME_SIZE as isize * MAX_FRAME_WINDOW_SIZE as isize);
+        assert_eq!(frames.len(), MAX_FRAME_WINDOW_SIZE as usize);
+
+        now_ms += MIN_SYNC_TIMEOUT_MS;
+
         let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
-        assert_eq!(frames.len(), 5);
+        assert_eq!(frames.len(), 1);
 
-        let frames = ta.emit_frames(now_ms + MIN_SYNC_TIMEOUT_MS, rtt_ms, 10000);
-        assert_eq!(frames.len(), 6);
-
-        test_sync_frame(&frames[0], frame::SyncFrame { next_frame_id: Some(5), next_packet_id: None });
-
-        let frames = ta.emit_frames(now_ms + 2*MIN_SYNC_TIMEOUT_MS, rtt_ms, 10000);
-        assert_eq!(frames.len(), 6);
-
-        test_sync_frame(&frames[0], frame::SyncFrame { next_frame_id: Some(10), next_packet_id: None });
+        test_sync_frame(&frames[0], frame::SyncFrame { next_frame_id: Some(MAX_FRAME_WINDOW_SIZE), next_packet_id: None });
     }
 
     // Sync case for which no the receiver has not yet called receive(), and only the packet window
     // is resynchronized.
     #[test]
     fn sync_packet_window_only() {
-        let now_ms = 0;
-        let rtt_ms = 100;
+        let rtt_ms = INITIAL_RTT_ESTIMATE_MS;
 
         let mut ta = TestApparatus::new();
+        let mut now_ms = 0;
 
         for _ in 0 .. 5 {
             ta.enqueue_packet(vec![ 0; MAX_FRAGMENT_SIZE ].into_boxed_slice(), 0, SendMode::Unreliable);
@@ -747,7 +805,9 @@ mod tests {
 
         ta.receive_ack(frame::AckFrame { frame_acks: Vec::new(), frame_window_base_id: 5, packet_window_base_id: 0 });
 
-        let frames = ta.emit_frames(now_ms + MIN_SYNC_TIMEOUT_MS, rtt_ms, 10000);
+        now_ms += MIN_SYNC_TIMEOUT_MS;
+
+        let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
         assert_eq!(frames.len(), 1);
 
         test_sync_frame(&frames[0], frame::SyncFrame { next_frame_id: None, next_packet_id: Some(5) });
@@ -757,7 +817,7 @@ mod tests {
     #[test]
     fn sync_response() {
         let now_ms = 0;
-        let rtt_ms = 100;
+        let rtt_ms = INITIAL_RTT_ESTIMATE_MS;
 
         let mut ta = TestApparatus::new();
 
@@ -769,42 +829,140 @@ mod tests {
         test_ack_frame(&frames[0], frame::AckFrame { frame_acks: Vec::new(), frame_window_base_id: 5, packet_window_base_id: 5 });
     }
 
-    // Keepalive syncs should be sent periodically after no data has been sent
+    // Packets should be resent [1, 2, 4, 4, ... 4] RTTs after the previous send.
     #[test]
-    fn sync_keepalive() {
-        let now_ms = 0;
-        let rtt_ms = 100;
+    fn resend_timing() {
+        let rtt_ms = INITIAL_RTT_ESTIMATE_MS;
 
         let mut ta = TestApparatus::new();
 
+        let p0 = (0 .. 400).map(|i| i as u8).collect::<Vec<u8>>().into_boxed_slice();
+        ta.enqueue_packet(p0.clone(), 0, SendMode::Persistent);
+
+        let frames = ta.emit_frames(0, rtt_ms, MAX_FRAME_SIZE as isize);
+        assert_eq!(frames.len(), 1);
+
+        let frames = ta.emit_frames(1, rtt_ms, MAX_FRAME_SIZE as isize);
+        assert_eq!(frames.len(), 0);
+
+        let resend_times = [ rtt_ms, 3*rtt_ms, 7*rtt_ms, 11*rtt_ms, 15*rtt_ms, 19*rtt_ms, 23*rtt_ms ];
+
+        for &now_ms in resend_times.iter() {
+            let frames = ta.emit_frames(now_ms - 1, rtt_ms, MAX_FRAME_SIZE as isize);
+            assert_eq!(frames.len(), 0);
+
+            let frames = ta.emit_frames(now_ms    , rtt_ms, MAX_FRAME_SIZE as isize);
+            assert_eq!(frames.len(), 1);
+
+            let frames = ta.emit_frames(now_ms + 1, rtt_ms, MAX_FRAME_SIZE as isize);
+            assert_eq!(frames.len(), 0);
+        }
+    }
+
+    // Keepalive syncs should be sent periodically after no data has been sent
+    #[test]
+    fn keepalive_timing() {
+        let rtt_ms = INITIAL_RTT_ESTIMATE_MS;
+
+        let mut ta = TestApparatus::new();
+        let mut now_ms = 0;
+
         let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
         assert_eq!(frames.len(), 0);
 
-        let frames = ta.emit_frames(now_ms + MIN_SYNC_TIMEOUT_MS - 1, rtt_ms, 10000);
+        now_ms += MIN_SYNC_KEEPALIVE_TIMEOUT_MS - 1;
+
+        let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
         assert_eq!(frames.len(), 0);
 
-        let frames = ta.emit_frames(now_ms + MIN_SYNC_TIMEOUT_MS, rtt_ms, 10000);
+        now_ms += 1;
+
+        let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
         assert_eq!(frames.len(), 1);
         test_sync_frame(&frames[0], frame::SyncFrame { next_frame_id: None, next_packet_id: None });
 
-        let frames = ta.emit_frames(now_ms + MIN_SYNC_TIMEOUT_MS, rtt_ms, 10000);
+        let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
         assert_eq!(frames.len(), 0);
 
-        let frames = ta.emit_frames(now_ms + 2 * MIN_SYNC_TIMEOUT_MS, rtt_ms, 10000);
+        now_ms += MIN_SYNC_KEEPALIVE_TIMEOUT_MS;
+
+        let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
         assert_eq!(frames.len(), 1);
         test_sync_frame(&frames[0], frame::SyncFrame { next_frame_id: None, next_packet_id: None });
 
         // Disrupt the normal timing
+        now_ms += MIN_SYNC_KEEPALIVE_TIMEOUT_MS/2;
+
         ta.enqueue_packet(vec![ 0; MAX_FRAGMENT_SIZE ].into_boxed_slice(), 0, SendMode::Unreliable);
-        let frames = ta.emit_frames(now_ms + 2 * MIN_SYNC_TIMEOUT_MS + MIN_SYNC_TIMEOUT_MS/2, rtt_ms, 10000);
+        let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
         assert_eq!(frames.len(), 1);
 
-        let frames = ta.emit_frames(now_ms + 3 * MIN_SYNC_TIMEOUT_MS, rtt_ms, 10000);
-        assert_eq!(frames.len(), 0);
+        now_ms += MIN_SYNC_TIMEOUT_MS;
 
-        let frames = ta.emit_frames(now_ms + 3 * MIN_SYNC_TIMEOUT_MS + MIN_SYNC_TIMEOUT_MS/2, rtt_ms, 10000);
+        let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
         assert_eq!(frames.len(), 1);
         test_sync_frame(&frames[0], frame::SyncFrame { next_frame_id: Some(1), next_packet_id: Some(1) });
+
+        ta.receive_ack(frame::AckFrame { frame_acks: Vec::new(), frame_window_base_id: 1, packet_window_base_id: 1 });
+
+        now_ms += MIN_SYNC_KEEPALIVE_TIMEOUT_MS;
+
+        let frames = ta.emit_frames(now_ms, rtt_ms, 10000);
+        assert_eq!(frames.len(), 1);
+        test_sync_frame(&frames[0], frame::SyncFrame { next_frame_id: None, next_packet_id: None });
+    }
+
+    // No two packets in a frame receive window may have the same ID
+    #[test]
+    fn packet_unambiguity() {
+        let now_ms = 0;
+        let rtt_ms = INITIAL_RTT_ESTIMATE_MS;
+
+        let mut ta = TestApparatus::new();
+
+        use frame::serial::{DATA_FRAME_OVERHEAD, DATA_FRAME_MAX_DATAGRAM_COUNT, MIN_DATAGRAM_OVERHEAD};
+        use crate::packet_id;
+
+        // Here we assume that sending hundreds of zero-length packets reaches the datagram limit,
+        // not the frame size limit.
+        assert!(DATA_FRAME_OVERHEAD + DATA_FRAME_MAX_DATAGRAM_COUNT * MIN_DATAGRAM_OVERHEAD <= MAX_FRAME_SIZE);
+
+        let mut max_id: usize = 0;
+
+        let receive_window_size = 2*MAX_FRAME_WINDOW_SIZE as usize;
+
+        for _ in 0 .. receive_window_size {
+            for _ in 0 .. DATA_FRAME_MAX_DATAGRAM_COUNT as usize {
+                ta.enqueue_packet(Vec::new().into_boxed_slice(), 0, SendMode::Unreliable);
+            }
+
+            let frames = ta.emit_frames(now_ms, rtt_ms, MAX_FRAME_SIZE as isize);
+            assert_eq!(frames.len(), 1);
+
+            use frame::serial::Serialize;
+            let data_frame = match frame::Frame::read(&frames[0]) {
+                Some(frame::Frame::DataFrame(data_frame)) => data_frame,
+                _ => panic!(),
+            };
+
+            for datagram in data_frame.datagrams.iter() {
+                // If the sequence IDs of packets wrap back around to zero, this will fail. If this
+                // does not fail, we do not wrap, and therefore have unique packets throughout the
+                // frame receive window.
+                assert_eq!(datagram.sequence_id as usize, max_id);
+                max_id += 1;
+            }
+
+            // Keep the windows happy
+            ta.receive_ack(frame::AckFrame {
+                frame_acks: Vec::new(),
+                frame_window_base_id: data_frame.sequence_id,
+                packet_window_base_id: max_id as u32,
+            });
+        }
+
+        println!("max_id: {}", max_id);
+        println!("packet_id::SPAN: {}", packet_id::SPAN);
     }
 }
 
