@@ -1,11 +1,10 @@
 
-mod fragment_buffer;
-
-use fragment_buffer::FragmentBuffer;
-
 use super::MAX_FRAGMENT_SIZE;
 use super::MAX_PACKET_WINDOW_SIZE;
-use super::Datagram;
+
+use crate::frame;
+
+mod fragment_buffer;
 
 struct ActiveEntry {
     // How many allocation points this entry is worth
@@ -18,7 +17,7 @@ struct ActiveEntry {
     last_fragment_id: u16,
 
     // Assembles a multi-fragment packet
-    asm_buffer: FragmentBuffer,
+    asm_buffer: fragment_buffer::FragmentBuffer,
 }
 
 impl ActiveEntry {
@@ -31,7 +30,7 @@ impl ActiveEntry {
             channel_parent_lead,
             last_fragment_id,
 
-            asm_buffer: FragmentBuffer::new(num_fragments),
+            asm_buffer: fragment_buffer::FragmentBuffer::new(num_fragments),
         }
     }
 }
@@ -51,8 +50,8 @@ pub struct Packet {
     pub data: Option<Box<[u8]>>,
 }
 
-fn packet_alloc_size(datagram: &Datagram) -> usize {
-    let num_fragments = datagram.fragment_id.last as usize + 1;
+fn packet_alloc_size(datagram: &frame::Datagram) -> usize {
+    let num_fragments = datagram.fragment_id_last as usize + 1;
     if num_fragments > 1 {
         num_fragments*MAX_FRAGMENT_SIZE
     } else {
@@ -69,9 +68,9 @@ pub struct AssemblyWindow {
 
 impl AssemblyWindow {
     pub fn new(max_alloc: usize) -> Self {
-        let max_alloc_ceil = (max_alloc + MAX_FRAGMENT_SIZE - 1)/MAX_FRAGMENT_SIZE*MAX_FRAGMENT_SIZE;
+        let window: Vec<WindowEntry> = (0 .. MAX_PACKET_WINDOW_SIZE).map(|_| WindowEntry::Open).collect();
 
-        let window: Vec<WindowEntry> = (0..MAX_PACKET_WINDOW_SIZE).map(|_| WindowEntry::Open).collect();
+        let max_alloc_ceil = ((max_alloc + MAX_FRAGMENT_SIZE - 1) / MAX_FRAGMENT_SIZE) * MAX_FRAGMENT_SIZE;
 
         Self {
             window: window.into_boxed_slice(),
@@ -81,8 +80,8 @@ impl AssemblyWindow {
         }
     }
 
-    pub fn try_add(&mut self, idx: usize, datagram: Datagram) -> Option<Packet> {
-        match &mut self.window[idx] {
+    pub fn try_add(&mut self, idx: usize, datagram: frame::Datagram) -> Option<Packet> {
+        match self.window[idx] {
             WindowEntry::Open => {
                 // New packet
 
@@ -107,7 +106,7 @@ impl AssemblyWindow {
                 } else {
                     self.alloc += alloc_size;
 
-                    if datagram.fragment_id.last == 0 {
+                    if datagram.fragment_id_last == 0 {
                         let new_entry = WindowEntry::Closed(alloc_size);
 
                         self.window[idx] = new_entry;
@@ -120,16 +119,16 @@ impl AssemblyWindow {
                             data: Some(datagram.data),
                         });
                     } else {
-                        let num_fragments = datagram.fragment_id.last as usize + 1;
+                        let num_fragments = datagram.fragment_id_last as usize + 1;
 
                         let mut new_entry = ActiveEntry::new(alloc_size,
                                                              datagram.channel_id,
                                                              datagram.window_parent_lead,
                                                              datagram.channel_parent_lead,
-                                                             datagram.fragment_id.last, 
+                                                             datagram.fragment_id_last,
                                                              num_fragments);
 
-                        new_entry.asm_buffer.write(datagram.fragment_id.id as usize, datagram.data);
+                        new_entry.asm_buffer.write(datagram.fragment_id as usize, datagram.data);
 
                         self.window[idx] = WindowEntry::Active(new_entry);
 
@@ -154,11 +153,11 @@ impl AssemblyWindow {
                 if datagram.channel_parent_lead != entry.channel_parent_lead {
                     return None;
                 }
-                if datagram.fragment_id.last != entry.last_fragment_id {
+                if datagram.fragment_id_last != entry.last_fragment_id {
                     return None;
                 }
 
-                entry.asm_buffer.write(datagram.fragment_id.id as usize, datagram.data);
+                entry.asm_buffer.write(datagram.fragment_id as usize, datagram.data);
 
                 if entry.asm_buffer.is_finished() {
                     let new_entry = WindowEntry::Closed(entry.alloc_size);
@@ -184,15 +183,13 @@ impl AssemblyWindow {
     }
 
     pub fn clear(&mut self, idx: usize) {
-        let ref mut window_entry = self.window[idx];
-
-        match window_entry {
+        match self.window[idx] {
             WindowEntry::Open => {
             }
             WindowEntry::Closed(ref alloc_size) => {
                 self.alloc -= alloc_size;
             }
-            WindowEntry::Active(entry) => {
+            WindowEntry::Active(ref entry) => {
                 self.alloc -= entry.alloc_size;
             }
         }
@@ -203,14 +200,9 @@ impl AssemblyWindow {
 
 #[cfg(test)]
 mod tests {
-    use crate::frame::Datagram;
-    use crate::frame::FragmentId;
+    use super::*;
 
-    use super::Packet;
-    use super::AssemblyWindow;
-    use super::MAX_FRAGMENT_SIZE;
-
-    use rand;
+    use frame::Datagram;
 
     #[test]
     fn single_packet_single_fragment() {
@@ -225,7 +217,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id: 0,
+            fragment_id_last: 0,
             data: packet_data.clone(),
         });
 
@@ -252,7 +245,8 @@ mod tests {
                 channel_id: 0,
                 window_parent_lead: 0,
                 channel_parent_lead: 0,
-                fragment_id: FragmentId { id: i as u16, last: 4 },
+                fragment_id: i as u16,
+                fragment_id_last: 4,
                 data: fragment_data,
             });
 
@@ -287,7 +281,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id: 0,
+            fragment_id_last: 0,
             data: packet_0_data.clone(),
         });
 
@@ -305,7 +300,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id: 0,
+            fragment_id_last: 0,
             data: vec![].into_boxed_slice(),
         });
 
@@ -324,7 +320,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 1 },
+            fragment_id: 0,
+            fragment_id_last: 1,
             data: packet_2_data[ .. MAX_FRAGMENT_SIZE].into(),
         });
 
@@ -341,7 +338,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 1, last: 1 },
+            fragment_id: 1,
+            fragment_id_last: 1,
             data: packet_2_data[MAX_FRAGMENT_SIZE .. ].into(),
         });
 
@@ -357,7 +355,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id: 0,
+            fragment_id_last: 0,
             data: packet_0_data.clone(),
         });
 
@@ -398,7 +397,8 @@ mod tests {
                 channel_id: 0,
                 window_parent_lead: 0,
                 channel_parent_lead: 0,
-                fragment_id: FragmentId { id: 0, last: (num_fragments - 1) as u16 },
+                fragment_id: 0,
+                fragment_id_last: (num_fragments - 1) as u16,
                 data: fragment_0_data,
             };
 
@@ -436,7 +436,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id: 0,
+            fragment_id_last: 0,
             data: packet_data.clone(),
         });
 
@@ -448,7 +449,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 2,
             channel_parent_lead: 1,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id: 0,
+            fragment_id_last: 0,
             data: packet_data.clone(),
         });
 
@@ -460,7 +462,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 1,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id: 0,
+            fragment_id_last: 0,
             data: packet_data.clone(),
         });
 
@@ -472,7 +475,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 1, last: 0 },
+            fragment_id: 1,
+            fragment_id_last: 0,
             data: packet_data.clone(),
         });
 
@@ -486,7 +490,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 1, last: 1 },
+            fragment_id: 1,
+            fragment_id_last: 1,
             data: packet_data.clone(),
         });
 
@@ -498,7 +503,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 1, last: 1 },
+            fragment_id: 1,
+            fragment_id_last: 1,
             data: packet_data[MAX_FRAGMENT_SIZE .. ].into(),
         });
 
@@ -510,7 +516,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 2, last: 1 },
+            fragment_id: 2,
+            fragment_id_last: 1,
             data: packet_data[ .. MAX_FRAGMENT_SIZE].into(),
         });
 
@@ -522,7 +529,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 1 },
+            fragment_id: 0,
+            fragment_id_last: 1,
             data: packet_data[ .. MAX_FRAGMENT_SIZE-1].into(),
         });
 
@@ -534,7 +542,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 1 },
+            fragment_id: 0,
+            fragment_id_last: 1,
             data: packet_data[ .. MAX_FRAGMENT_SIZE].into(),
         });
 
@@ -560,7 +569,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 1 },
+            fragment_id: 0,
+            fragment_id_last: 1,
             data: packet_data[ .. MAX_FRAGMENT_SIZE].into(),
         });
 
@@ -572,7 +582,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 0, last: 1 },
+            fragment_id: 0,
+            fragment_id_last: 1,
             data: packet_data[ .. MAX_FRAGMENT_SIZE].into(),
         });
 
@@ -584,7 +595,8 @@ mod tests {
             channel_id: 1,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 1, last: 1 },
+            fragment_id: 1,
+            fragment_id_last: 1,
             data: packet_data[MAX_FRAGMENT_SIZE .. ].into(),
         });
 
@@ -596,7 +608,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 1,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 1, last: 1 },
+            fragment_id: 1,
+            fragment_id_last: 1,
             data: packet_data[MAX_FRAGMENT_SIZE .. ].into(),
         });
 
@@ -608,7 +621,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 1,
-            fragment_id: FragmentId { id: 1, last: 1 },
+            fragment_id: 1,
+            fragment_id_last: 1,
             data: packet_data[MAX_FRAGMENT_SIZE .. ].into(),
         });
 
@@ -620,7 +634,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 1, last: 2 },
+            fragment_id: 1,
+            fragment_id_last: 2,
             data: packet_data[MAX_FRAGMENT_SIZE .. ].into(),
         });
 
@@ -632,7 +647,8 @@ mod tests {
             channel_id: 0,
             window_parent_lead: 0,
             channel_parent_lead: 0,
-            fragment_id: FragmentId { id: 1, last: 1 },
+            fragment_id: 1,
+            fragment_id_last: 1,
             data: packet_data[MAX_FRAGMENT_SIZE .. ].into(),
         });
 
