@@ -1,10 +1,11 @@
 
 use super::pending_packet::{PendingPacket, PendingPacketRc};
 
-use crate::MAX_CHANNELS;
+use crate::CHANNEL_COUNT;
 use crate::MAX_FRAGMENT_SIZE;
 use crate::MAX_PACKET_SIZE;
 use crate::MAX_PACKET_WINDOW_SIZE;
+use crate::packet_id;
 use crate::SendMode;
 
 use std::collections::VecDeque;
@@ -21,8 +22,7 @@ fn alloc_size(packet_size: usize) -> usize {
 }
 
 struct WindowEntry {
-    // The packet to be sent in this slot (never read, only deleted once window advances)
-    #[allow(dead_code)]
+    // The packet to be sent in this slot
     packet: PendingPacketRc,
     // How many allocation points this packet is worth
     alloc_size: usize,
@@ -61,29 +61,43 @@ impl PacketSendEntry {
     }
 }
 
+macro_rules! window_index {
+    ($self:ident, $sequence_id:expr) => {
+        ($sequence_id & $self.window_mask) as usize
+    };
+}
+
 pub struct PacketSender {
     packet_send_queue: VecDeque<PacketSendEntry>,
 
     base_id: u32,
     next_id: u32,
     window: Box<[Option<WindowEntry>]>,
+    window_size: u32,
+    window_mask: u32,
+
+    window_parent_id: Option<u32>,
+    channels: Box<[Channel]>,
 
     max_alloc: usize,
     alloc: usize,
 
-    parent_id: Option<u32>,
-    channels: Box<[Channel]>,
+    total_size: usize,
 }
 
 impl PacketSender {
-    pub fn new(channel_num: usize, max_alloc: usize, base_id: u32) -> Self {
-        debug_assert!(channel_num > 0);
-        debug_assert!(channel_num <= MAX_CHANNELS);
+    pub fn new(window_size: u32, base_id: u32, max_alloc: usize) -> Self {
+        debug_assert!(window_size > 0);
+        debug_assert!(window_size <= MAX_PACKET_WINDOW_SIZE);
+        debug_assert!(window_size & (window_size - 1) == 0);
 
-        let max_alloc_ceil = (max_alloc + MAX_FRAGMENT_SIZE - 1)/MAX_FRAGMENT_SIZE*MAX_FRAGMENT_SIZE;
+        debug_assert!(packet_id::is_valid(base_id));
 
-        let window: Vec<Option<WindowEntry>> = (0..MAX_PACKET_WINDOW_SIZE).map(|_| None).collect();
-        let channels: Vec<Channel> = (0..channel_num).map(|_| Channel::new()).collect();
+        let window: Vec<Option<WindowEntry>> = (0 .. window_size).map(|_| None).collect();
+
+        let channels: Vec<Channel> = (0 .. CHANNEL_COUNT).map(|_| Channel::new()).collect();
+
+        let max_alloc_ceil = ((max_alloc + MAX_FRAGMENT_SIZE - 1) / MAX_FRAGMENT_SIZE) * MAX_FRAGMENT_SIZE;
 
         Self {
             packet_send_queue: VecDeque::new(),
@@ -91,12 +105,16 @@ impl PacketSender {
             base_id: base_id,
             next_id: base_id,
             window: window.into_boxed_slice(),
+            window_size: window_size,
+            window_mask: window_size - 1,
+
+            window_parent_id: None,
+            channels: channels.into_boxed_slice(),
 
             max_alloc: max_alloc_ceil,
             alloc: 0,
 
-            parent_id: None,
-            channels: channels.into_boxed_slice(),
+            total_size: 0,
         }
     }
 
@@ -104,6 +122,10 @@ impl PacketSender {
         self.packet_send_queue.len()
     }
 
+    pub fn total_size(&self) -> usize {
+        self.total_size
+    }
+
     pub fn next_id(&self) -> u32 {
         self.next_id
     }
@@ -112,13 +134,13 @@ impl PacketSender {
         self.base_id
     }
 
-    // Places a user packet on the send queue. Silently drops the packet if the packet exceeds the
-    // receiver's maximum receive allocation.
+    // Places a user packet on the send queue.
     pub fn enqueue_packet(&mut self, data: Box<[u8]>, channel_id: u8, mode: SendMode, flush_id: u32) {
         debug_assert!(data.len() <= MAX_PACKET_SIZE);
         debug_assert!(data.len() <= self.max_alloc);
-        debug_assert!((channel_id as usize) < self.channels.len());
+        debug_assert!((channel_id as usize) < CHANNEL_COUNT);
 
+        self.total_size += data.len();
         self.packet_send_queue.push_back(PacketSendEntry::new(data, channel_id, mode, flush_id));
     }
 
@@ -129,6 +151,7 @@ impl PacketSender {
             match packet.mode {
                 SendMode::TimeSensitive => {
                     if packet.flush_id != flush_id {
+                        self.total_size -= packet.data.len();
                         self.packet_send_queue.pop_front();
                     } else {
                         break;
@@ -139,7 +162,7 @@ impl PacketSender {
         }
 
         if let Some(packet) = self.packet_send_queue.front() {
-            if self.next_id.wrapping_sub(self.base_id) >= MAX_PACKET_WINDOW_SIZE {
+            if packet_id::sub(self.next_id, self.base_id) >= self.window_size {
                 return None;
             }
 
@@ -155,8 +178,8 @@ impl PacketSender {
             let ref mut channel = self.channels[packet.channel_id as usize];
 
             let window_parent_lead =
-                if let Some(parent_id) = self.parent_id {
-                    let lead = sequence_id.wrapping_sub(parent_id);
+                if let Some(parent_id) = self.window_parent_id {
+                    let lead = packet_id::sub(sequence_id, parent_id);
                     debug_assert!(lead <= u16::MAX as u32);
                     lead as u16
                 } else {
@@ -165,24 +188,13 @@ impl PacketSender {
 
             let channel_parent_lead =
                 if let Some(parent_id) = channel.parent_id {
-                    let lead = sequence_id.wrapping_sub(parent_id);
+                    let lead = packet_id::sub(sequence_id, parent_id);
                     debug_assert!(lead <= u16::MAX as u32);
                     lead as u16
                 } else {
                     0
                 };
 
-            match packet.mode {
-                SendMode::Reliable => {
-                    self.parent_id = Some(sequence_id);
-                    channel.parent_id = Some(sequence_id);
-                }
-                _ => ()
-            }
-
-            self.next_id = self.next_id.wrapping_add(1);
-            self.alloc += packet_alloc_size;
-
             let pending_packet = Rc::new(RefCell::new(PendingPacket::new(packet.data,
                                                                          packet.channel_id,
                                                                          sequence_id,
@@ -191,15 +203,27 @@ impl PacketSender {
 
             let pending_packet_clone = Rc::clone(&pending_packet);
 
-            let window_idx = (sequence_id % MAX_PACKET_WINDOW_SIZE) as usize;
-            debug_assert!(self.window[window_idx].is_none());
+            let window_idx = window_index!(self, sequence_id);
 
+            debug_assert!(self.window[window_idx].is_none());
             self.window[window_idx] = Some(WindowEntry {
                 packet: pending_packet,
                 alloc_size: packet_alloc_size,
                 channel_id: packet.channel_id
             });
 
+            self.next_id = packet_id::add(self.next_id, 1);
+
+            self.alloc += packet_alloc_size;
+
+            match packet.mode {
+                SendMode::Reliable => {
+                    self.window_parent_id = Some(sequence_id);
+                    channel.parent_id = Some(sequence_id);
+                }
+                _ => ()
+            }
+
             let resend = match packet.mode {
                 SendMode::TimeSensitive => false,
                 SendMode::Unreliable => false,
@@ -216,41 +240,37 @@ impl PacketSender {
     // Responds to a receive window acknowledgement. All packet data beyond the new receive window
     // is forgotten, thereby freeing transfer window & allocation space for new packets.
     pub fn acknowledge(&mut self, receiver_base_id: u32) {
-        let window_size = self.next_id.wrapping_sub(self.base_id);
-        let ack_delta = receiver_base_id.wrapping_sub(self.base_id);
+        let receiver_delta = packet_id::sub(receiver_base_id, self.base_id);
+        let span = packet_id::sub(self.next_id, self.base_id);
 
-        if ack_delta > window_size {
+        if receiver_delta > span {
             return;
         }
 
         while self.base_id != receiver_base_id {
-            let window_idx = (self.base_id % MAX_PACKET_WINDOW_SIZE) as usize;
+            let window_idx = window_index!(self, self.base_id);
+            let ref mut entry = self.window[window_idx].as_ref().unwrap();
 
-            let ref mut window_entry = self.window[window_idx];
+            let ref mut channel = self.channels[entry.channel_id as usize];
 
-            if let Some(entry) = window_entry {
-                let ref mut channel = self.channels[entry.channel_id as usize];
-
-                if let Some(parent_id) = self.parent_id {
-                    if parent_id == self.base_id {
-                        self.parent_id = None;
-                    }
+            if let Some(parent_id) = self.window_parent_id {
+                if parent_id == self.base_id {
+                    self.window_parent_id = None;
                 }
-
-                if let Some(parent_id) = channel.parent_id {
-                    if parent_id == self.base_id {
-                        channel.parent_id = None;
-                    }
-                }
-
-                self.alloc -= entry.alloc_size;
-
-                *window_entry = None;
-            } else {
-                panic!();
             }
 
-            self.base_id = self.base_id.wrapping_add(1);
+            if let Some(parent_id) = channel.parent_id {
+                if parent_id == self.base_id {
+                    channel.parent_id = None;
+                }
+            }
+
+            self.alloc -= entry.alloc_size;
+            self.total_size -= entry.packet.borrow().size();
+
+            self.window[window_idx] = None;
+
+            self.base_id = packet_id::add(self.base_id, 1);
         }
     }
 }
@@ -286,7 +306,7 @@ mod tests {
 
     #[test]
     fn basic() {
-        let mut tx = PacketSender::new(1, 10000, 0);
+        let mut tx = PacketSender::new(MAX_PACKET_WINDOW_SIZE, 0, 10000);
 
         tx.enqueue_packet(new_packet_data(0), 0, SendMode::TimeSensitive, 0);
         tx.enqueue_packet(new_packet_data(1), 0, SendMode::Unreliable, 0);
@@ -315,7 +335,7 @@ mod tests {
               w  c0 c1
         */
 
-        let mut tx = PacketSender::new(2, 10000, 0);
+        let mut tx = PacketSender::new(MAX_PACKET_WINDOW_SIZE, 0, 10000);
 
         tx.enqueue_packet(new_packet_data(0), 1, SendMode::Unreliable, 0);
         tx.enqueue_packet(new_packet_data(1), 1, SendMode::Reliable, 0);
@@ -355,7 +375,7 @@ mod tests {
               w  c0 c1
         */
 
-        let mut tx = PacketSender::new(2, 10000, 0);
+        let mut tx = PacketSender::new(MAX_PACKET_WINDOW_SIZE, 0, 10000);
 
         let mut flush_id = 0;
 
@@ -386,7 +406,7 @@ mod tests {
 
             assert!(tx.emit_packet(0).is_none());
 
-            flush_id = flush_id.wrapping_add(1);
+            flush_id = packet_id::add(flush_id, 1);
         }
     }
 
