@@ -1,21 +1,22 @@
 
-mod daten_meister;
-
-use crate::MAX_PACKET_SIZE;
-use crate::MAX_CHANNELS;
-use crate::PROTOCOL_VERSION;
-use crate::frame;
-use crate::SendMode;
+use crate::CHANNEL_COUNT;
 use crate::Event;
+use crate::MAX_FRAME_WINDOW_SIZE;
+use crate::MAX_PACKET_SIZE;
+use crate::MAX_PACKET_WINDOW_SIZE;
+use crate::PROTOCOL_VERSION;
+use crate::SendMode;
+use crate::frame;
 
 use frame::serial::Serialize;
-use daten_meister::DatenMeister;
 
 use rand;
 
 use std::time;
 use std::collections::VecDeque;
 
+mod daten_meister;
+
 pub trait FrameSink {
     fn send(&mut self, frame_data: &[u8]);
 }
@@ -23,24 +24,16 @@ pub trait FrameSink {
 /// Parameters used to configure either endpoint of a `uflow` connection.
 #[derive(Clone,Debug)]
 pub struct Config {
-    /// The number of channels used to send packets.
-    ///
-    /// Must be greater than 0, and less than or equal to [`MAX_CHANNELS`].
-    ///
-    /// *Note*: The number of channels used by an endpoint to send data may differ from the
-    /// opposing endpoint.
-    pub channel_count: usize,
-
     /// The maximum send rate, in bytes per second. The endpoint will ensure that its outgoing
     /// bandwidth does not exceed this value.
     ///
-    /// Must be greater than 0. Values larger than `2^32` will be truncated.
+    /// Must be greater than 0. Values larger than 2^32 will be truncated.
     pub max_send_rate: usize,
 
     /// The maximum acceptable receive rate, in bytes per second. The opposing endpoint will ensure
     /// that its outgoing bandwidth does not exceed this value.
     ///
-    /// Must be greater than 0. Values larger than `2^32` will be truncated.
+    /// Must be greater than 0. Values larger than 2^32 will be truncated.
     pub max_receive_rate: usize,
 
     /// The maximum size of a sent packet, in bytes. The endpoint will ensure that it does not send
@@ -60,54 +53,36 @@ pub struct Config {
     /// size. A connection attempt will fail if the `max_packet_size` of the opposing endpoint
     /// exceeds this value.
     pub max_receive_alloc: usize,
+
+    /// Whether the endpoint should automatically send keepalive frames if no data has been sent
+    /// for one keepalive interval (currently 5 seconds). If set to false, the connection will time
+    /// out if either endpoint does not send data for one timeout interval (currently 20 seconds).
+    pub keepalive: bool,
 }
 
 impl Default for Config {
     /// Creates an endpoint configuration with the following parameters:
-    ///   * Number of channels: 1
-    ///   * Maximum outgoing bandwidth: 10MB/s
-    ///   * Maximum incoming bandwidth: 10MB/s
-    ///   * Maximum receive allocation: 1MB
+    ///   * Maximum outgoing bandwidth: 2MB/s
+    ///   * Maximum incoming bandwidth: 2MB/s
+    ///   * Maximum packet size: 1MB
+    ///   * Maximum packet receive allocation: 1MB
+    ///   * Keepalive: true
     fn default() -> Self {
         Self {
-            channel_count: 1,
-            max_send_rate: 10_000_000,
-            max_receive_rate: 10_000_000,
+            max_send_rate: 2_000_000,
+            max_receive_rate: 2_000_000,
+
             max_packet_size: 1_000_000,
             max_receive_alloc: 1_000_000,
+
+            keepalive: true,
         }
     }
 }
 
 impl Config {
-    /// Sets `channel_count` to the provided value.
-    pub fn channel_count(mut self, channel_count: usize) -> Config {
-        self.channel_count = channel_count;
-        self
-    }
-
-    /// Sets `max_send_rate` to the provided value.
-    pub fn max_send_rate(mut self, rate: usize) -> Config {
-        self.max_send_rate = rate;
-        self
-    }
-
-    /// Sets `max_receive_rate` to the provided value.
-    pub fn max_receive_rate(mut self, rate: usize) -> Config {
-        self.max_receive_rate = rate;
-        self
-    }
-
-    /// Sets `max_receive_alloc` to the provided value.
-    pub fn max_receive_alloc(mut self, max_receive_alloc: usize) -> Config {
-        self.max_receive_alloc = max_receive_alloc;
-        self
-    }
-
     /// Returns `true` if each parameter has a valid value.
     pub fn is_valid(&self) -> bool {
-        self.channel_count > 0 &&
-        self.channel_count <= MAX_CHANNELS &&
         self.max_send_rate > 0 &&
         self.max_receive_rate > 0 &&
         self.max_packet_size > 0 &&
@@ -135,7 +110,7 @@ struct ConnectingState {
 }
 
 struct ConnectedState {
-    daten_meister: DatenMeister,
+    daten_meister: daten_meister::DatenMeister,
 
     disconnect_flush: bool,
 }
@@ -185,8 +160,8 @@ pub struct Endpoint {
 
     watchdog_time: time::Instant,
 
-    channel_count: usize,
     max_packet_size: usize,
+    keepalive: bool,
     was_connected: bool,
 }
 
@@ -196,10 +171,12 @@ impl Endpoint {
 
         let connect_frame = frame::ConnectFrame {
             version: PROTOCOL_VERSION,
-            channel_count_sup: (cfg.channel_count - 1) as u8,
+
             max_receive_rate: cfg.max_receive_rate.min(u32::MAX as usize) as u32,
+
             max_packet_size: cfg.max_packet_size.min(u32::MAX as usize) as u32,
             max_receive_alloc: cfg.max_receive_alloc.min(u32::MAX as usize) as u32,
+
             nonce: rand::random::<u32>(),
         };
 
@@ -221,19 +198,22 @@ impl Endpoint {
 
             watchdog_time: time::Instant::now(),
 
-            channel_count: cfg.channel_count,
             max_packet_size: cfg.max_packet_size,
+            keepalive: cfg.keepalive,
+
             was_connected: false,
         }
     }
 
     pub fn send(&mut self, data: Box<[u8]>, channel_id: usize, mode: SendMode) {
         assert!(data.len() <= self.max_packet_size,
-                "send failed: packet of size {} exceeds maximum size {}",
+                "send failed: packet of size {} exceeds configured maximum of {}",
                 data.len(),
                 self.max_packet_size);
 
-        assert!(channel_id < self.channel_count, "send failed: invalid channel ID {}", channel_id);
+        assert!((channel_id as usize) < CHANNEL_COUNT,
+                "send failed: channel ID {} is invalid",
+                channel_id);
 
         match self.state {
             State::Connecting(ref mut state) => {
@@ -252,6 +232,7 @@ impl Endpoint {
                 self.enter_disconnecting();
             }
             State::Connected(ref mut state) => {
+                // XXX TODO: Need a timeout here
                 state.disconnect_flush = true;
             }
             State::Disconnecting(_) => {
@@ -281,10 +262,9 @@ impl Endpoint {
         }
     }
 
-    fn validate_handshake(remote_info: &frame::ConnectFrame, max_packet_size: usize) -> bool {
-        remote_info.version == PROTOCOL_VERSION &&
-        remote_info.channel_count_sup as usize <= MAX_CHANNELS - 1 &&
-        remote_info.max_receive_alloc as usize >= max_packet_size
+    fn validate_handshake(connect_frame_remote: &frame::ConnectFrame, max_packet_size: usize) -> bool {
+        connect_frame_remote.version == PROTOCOL_VERSION &&
+        connect_frame_remote.max_receive_alloc as usize >= max_packet_size
     }
 
     pub fn handle_frame(&mut self, frame: frame::Frame, sink: &mut impl FrameSink) {
@@ -386,47 +366,29 @@ impl Endpoint {
         }
     }
 
-    pub fn flush(&mut self, sink: &mut impl FrameSink) {
+    pub fn step(&mut self) {
         let now = time::Instant::now();
 
         match self.state {
-            State::Connecting(ref mut state) => {
+            State::Connecting(_) => {
                 if now - self.watchdog_time > CONNECTING_WATCHDOG_TIMEOUT {
                     self.enter_zombie();
                     return;
                 }
-
-                if !state.connect_ack_received {
-                    if state.last_send_time.map_or(true, |time| now - time > CONNECT_INTERVAL) {
-                        state.last_send_time = Some(now);
-                        sink.send(&frame::Frame::ConnectFrame(state.connect_frame.clone()).write());
-                    }
-                }
             }
             State::Connected(ref mut state) => {
+                state.daten_meister.step();
+
                 if now - self.watchdog_time > CONNECTED_WATCHDOG_TIMEOUT {
                     self.enter_zombie();
                     return;
                 }
-
-                state.daten_meister.flush(sink);
-
-                if state.disconnect_flush {
-                    if !state.daten_meister.is_send_pending() {
-                        self.enter_disconnecting();
-                    }
-                }
             }
-            State::Disconnecting(ref mut state) => {
+            State::Disconnecting(_) => {
                 if now - self.watchdog_time > DISCONNECTING_WATCHDOG_TIMEOUT {
                     self.enter_zombie();
                     return;
                 }
-
-                if state.last_send_time.map_or(true, |time| now - time > DISCONNECT_INTERVAL) {
-                    state.last_send_time = Some(now);
-                    sink.send(&frame::Frame::DisconnectFrame(frame::DisconnectFrame { }).write());
-                }
             }
             State::Disconnected => {
                 if now - self.watchdog_time > DISCONNECTED_WATCHDOG_TIMEOUT {
@@ -439,6 +401,38 @@ impl Endpoint {
         }
     }
 
+    pub fn flush(&mut self, sink: &mut impl FrameSink) {
+        let now = time::Instant::now();
+
+        match self.state {
+            State::Connecting(ref mut state) => {
+                if !state.connect_ack_received {
+                    if state.last_send_time.map_or(true, |time| now - time > CONNECT_INTERVAL) {
+                        state.last_send_time = Some(now);
+                        sink.send(&frame::Frame::ConnectFrame(state.connect_frame.clone()).write());
+                    }
+                }
+            }
+            State::Connected(ref mut state) => {
+                state.daten_meister.flush(sink);
+
+                if state.disconnect_flush {
+                    if !state.daten_meister.is_send_pending() {
+                        self.enter_disconnecting();
+                        return;
+                    }
+                }
+            }
+            State::Disconnecting(ref mut state) => {
+                if state.last_send_time.map_or(true, |time| now - time > DISCONNECT_INTERVAL) {
+                    state.last_send_time = Some(now);
+                    sink.send(&frame::Frame::DisconnectFrame(frame::DisconnectFrame { }).write());
+                }
+            }
+            _ => (),
+        }
+    }
+
     pub fn poll_events(&mut self) -> impl Iterator<Item = Event> {
         match self.state {
             State::Connected(ref mut state) => {
@@ -471,23 +465,42 @@ impl Endpoint {
         }
     }
 
+    pub fn send_buffer_size(&self) -> usize {
+        match self.state {
+            State::Connected(ref state) => state.daten_meister.send_buffer_size(),
+            _ => 0
+        }
+    }
+
     fn enter_connected(&mut self,
                        local: frame::ConnectFrame,
                        remote: frame::ConnectFrame,
                        initial_sends: VecDeque<SendEntry>,
                        max_send_rate: u32) {
-        let tx_channels = local.channel_count_sup as usize + 1;
-        let rx_channels = remote.channel_count_sup as usize + 1;
-        let tx_alloc_limit = remote.max_receive_alloc as usize;
-        let rx_alloc_limit = local.max_receive_alloc as usize;
-        let tx_base_id = local.nonce;
-        let rx_base_id = remote.nonce;
-        let tx_bandwidth_limit = max_send_rate.min(remote.max_receive_rate);
+        use crate::packet_id;
 
-        let mut daten_meister = DatenMeister::new(tx_channels, rx_channels,
-                                                  tx_alloc_limit, rx_alloc_limit,
-                                                  tx_base_id, rx_base_id,
-                                                  tx_bandwidth_limit);
+        let config = daten_meister::Config {
+            tx_frame_window_size: MAX_FRAME_WINDOW_SIZE,
+            rx_frame_window_size: MAX_FRAME_WINDOW_SIZE,
+
+            tx_frame_base_id: local.nonce,
+            rx_frame_base_id: remote.nonce,
+
+            tx_packet_window_size: MAX_PACKET_WINDOW_SIZE,
+            rx_packet_window_size: MAX_PACKET_WINDOW_SIZE,
+
+            tx_packet_base_id: local.nonce & packet_id::MASK,
+            rx_packet_base_id: remote.nonce & packet_id::MASK,
+
+            tx_bandwidth_limit: max_send_rate.min(remote.max_receive_rate),
+
+            tx_alloc_limit: remote.max_receive_alloc as usize,
+            rx_alloc_limit: local.max_receive_alloc as usize,
+
+            keepalive: self.keepalive,
+        };
+
+        let mut daten_meister = daten_meister::DatenMeister::new(config);
 
         for send in initial_sends.into_iter() {
             daten_meister.send(send.data, send.channel_id, send.mode);
