@@ -1,12 +1,15 @@
 
+use super::*;
+
 mod build;
+mod crc;
 
 pub use build::DataFrameBuilder;
 pub use build::AckFrameBuilder;
 
-use super::*;
-
 const FRAME_HEADER_SIZE: usize = 1;
+const FRAME_CRC_SIZE: usize = 4;
+const FRAME_OVERHEAD: usize = FRAME_HEADER_SIZE + FRAME_CRC_SIZE;
 
 const CONNECT_FRAME_ID: u8 = 0;
 const CONNECT_ACK_FRAME_ID: u8 = 1;
@@ -16,7 +19,7 @@ const DATA_FRAME_ID: u8 = 4;
 const SYNC_FRAME_ID: u8 = 5;
 const ACK_FRAME_ID: u8 = 6;
 
-const CONNECT_FRAME_PAYLOAD_SIZE: usize = 18;
+const CONNECT_FRAME_PAYLOAD_SIZE: usize = 17;
 
 const CONNECT_ACK_FRAME_PAYLOAD_SIZE: usize = 4;
 
@@ -24,18 +27,24 @@ const DISCONNECT_FRAME_PAYLOAD_SIZE: usize = 0;
 
 const DISCONNECT_ACK_FRAME_PAYLOAD_SIZE: usize = 0;
 
-const DATAGRAM_HEADER_SIZE_FRAGMENT: usize = 15;
-const DATAGRAM_HEADER_SIZE_FULL: usize = 11;
-const DATA_FRAME_PAYLOAD_HEADER_SIZE: usize = 7;
-pub const MAX_DATAGRAM_OVERHEAD: usize = DATAGRAM_HEADER_SIZE_FRAGMENT;
-pub const DATA_FRAME_OVERHEAD: usize = FRAME_HEADER_SIZE + DATA_FRAME_PAYLOAD_HEADER_SIZE;
+const DATAGRAM_HEADER_SIZE_MICRO: usize =  6;
+const DATAGRAM_HEADER_SIZE_SMALL: usize =  9;
+const DATAGRAM_HEADER_SIZE_LARGE: usize = 14;
+const DATAGRAM_HEADER_SIZE_MIN: usize =  DATAGRAM_HEADER_SIZE_MICRO;
+pub const MAX_DATAGRAM_OVERHEAD: usize = DATAGRAM_HEADER_SIZE_LARGE;
+#[cfg(test)]
+pub const MIN_DATAGRAM_OVERHEAD: usize = DATAGRAM_HEADER_SIZE_MICRO;
+
+const DATA_FRAME_PAYLOAD_HEADER_SIZE: usize = 5;
+pub const DATA_FRAME_OVERHEAD: usize = FRAME_OVERHEAD + DATA_FRAME_PAYLOAD_HEADER_SIZE;
+pub const DATA_FRAME_MAX_DATAGRAM_COUNT: usize = 127;
 
 const SYNC_FRAME_PAYLOAD_SIZE: usize = 9;
-pub const SYNC_FRAME_SIZE: usize = FRAME_HEADER_SIZE + SYNC_FRAME_PAYLOAD_SIZE;
 
-const ACK_GROUP_SIZE: usize = 9;
+pub const ACK_GROUP_SIZE: usize = 9;
 const ACK_FRAME_PAYLOAD_HEADER_SIZE: usize = 10;
-pub const ACK_FRAME_OVERHEAD: usize = FRAME_HEADER_SIZE + ACK_FRAME_PAYLOAD_HEADER_SIZE;
+#[cfg(test)]
+pub const ACK_FRAME_OVERHEAD: usize = FRAME_OVERHEAD + ACK_FRAME_PAYLOAD_HEADER_SIZE;
 
 pub const MAX_CHANNELS: usize = 64;
 pub const MAX_FRAGMENTS: usize = 1 << 16;
@@ -52,28 +61,27 @@ fn read_connect_payload(data: &[u8]) -> Option<Frame> {
                 ((data[3] as u32) <<  8) |
                 ((data[4] as u32)      );
 
-    let channel_count_sup = data[5];
+    let max_receive_rate = ((data[5] as u32) << 24) |
+                           ((data[6] as u32) << 16) |
+                           ((data[7] as u32) <<  8) |
+                           ((data[8] as u32)      );
 
-    let max_receive_rate = ((data[6] as u32) << 24) |
-                           ((data[7] as u32) << 16) |
-                           ((data[8] as u32) <<  8) |
-                           ((data[9] as u32)      );
+    let max_packet_size = ((data[9] as u32) << 24) |
+                          ((data[10] as u32) << 16) |
+                          ((data[11] as u32) <<  8) |
+                          ((data[12] as u32)      );
 
-    let max_packet_size = ((data[10] as u32) << 24) |
-                          ((data[11] as u32) << 16) |
-                          ((data[12] as u32) <<  8) |
-                          ((data[13] as u32)      );
-
-    let max_receive_alloc = ((data[14] as u32) << 24) |
-                            ((data[15] as u32) << 16) |
-                            ((data[16] as u32) <<  8) |
-                            ((data[17] as u32)      );
+    let max_receive_alloc = ((data[13] as u32) << 24) |
+                            ((data[14] as u32) << 16) |
+                            ((data[15] as u32) <<  8) |
+                            ((data[16] as u32)      );
 
     Some(Frame::ConnectFrame(ConnectFrame {
         version,
         nonce,
-        channel_count_sup,
+
         max_receive_rate,
+
         max_packet_size,
         max_receive_alloc,
     }))
@@ -109,46 +117,37 @@ fn read_disconnect_ack_payload(data: &[u8]) -> Option<Frame> {
 }
 
 fn read_datagram(data: &[u8]) -> Option<(Datagram, usize)> {
-    if data.len() < 1 {
+    if data.len() < DATAGRAM_HEADER_SIZE_MIN {
         return None;
     }
 
-    let header_byte = data[0];
+    if data[0] & 0x80 == 0x00 {
+        let header_size = DATAGRAM_HEADER_SIZE_MICRO;
 
-    if header_byte & 0x80 == 0x80 {
-        // Fragment
-        let header_size = DATAGRAM_HEADER_SIZE_FRAGMENT;
+        let data_len = (data[0] & 0x3F) as usize;
 
-        if data.len() < header_size {
+        let total_size = header_size + data_len;
+
+        if data.len() < total_size {
             return None;
         }
 
-        let channel_id = header_byte & 0x3F;
+        // Micro
+        let channel_id = ((data[4] >> 2) & 0x20) |
+                         ((data[0] >> 2) & 0x10) |
+                         ((data[1]     ) & 0x0F);
 
-        let sequence_id = ((data[1] as u32) << 24) |
-                          ((data[2] as u32) << 16) |
-                          ((data[3] as u32) <<  8) |
-                          ((data[4] as u32)      );
+        let sequence_id = (((data[1] & 0xF0) as u32) << 12) |
+                          (((data[2]       ) as u32) <<  8) |
+                          (((data[3]       ) as u32)      );
 
-        let window_parent_lead = ((data[5] as u16) << 8) |
-                                 ((data[6] as u16)     );
+        let window_parent_lead = (data[4] & 0x7F) as u16;
 
-        let channel_parent_lead = ((data[7] as u16) << 8) |
-                                  ((data[8] as u16)     );
+        let channel_parent_lead = data[5] as u16;
 
-        let fragment_id_last = ((data[ 9] as u16) << 8) |
-                               ((data[10] as u16)     );
+        let fragment_id = 0;
 
-        let fragment_id = ((data[11] as u16) << 8) |
-                          ((data[12] as u16)     );
-
-        let data_len_u16 = ((data[13] as u16) << 8) |
-                           ((data[14] as u16)     );
-        let data_len = data_len_u16 as usize;
-
-        if data.len() < header_size + data_len {
-            return None;
-        }
+        let fragment_id_last = 0;
 
         let data = data[header_size .. header_size + data_len].into();
 
@@ -157,37 +156,79 @@ fn read_datagram(data: &[u8]) -> Option<(Datagram, usize)> {
             sequence_id,
             window_parent_lead,
             channel_parent_lead,
-            fragment_id: FragmentId { id: fragment_id, last: fragment_id_last },
+            fragment_id,
+            fragment_id_last,
             data,
-        }, header_size + data_len));
+        }, total_size));
+    } else if data[0] & 0x40 == 0x00 {
+        // Small
+        let header_size = DATAGRAM_HEADER_SIZE_SMALL;
+
+        let data_len = data[1] as usize;
+
+        let total_size = header_size + data_len;
+
+        if data.len() < total_size {
+            return None;
+        }
+
+        let channel_id = data[0] & 0x3F;
+
+        let sequence_id = (((data[2] & 0x0F) as u32) << 16) |
+                          (((data[3]       ) as u32) <<  8) |
+                          (((data[4]       ) as u32)      );
+
+        let window_parent_lead = ((data[5] as u16) << 8) |
+                                 ((data[6] as u16)     );
+
+        let channel_parent_lead = ((data[7] as u16) << 8) |
+                                  ((data[8] as u16)     );
+
+        let fragment_id = 0;
+
+        let fragment_id_last = 0;
+
+        let data = data[header_size .. header_size + data_len].into();
+
+        return Some((Datagram {
+            channel_id,
+            sequence_id,
+            window_parent_lead,
+            channel_parent_lead,
+            fragment_id,
+            fragment_id_last,
+            data,
+        }, total_size));
     } else {
-        // Full
-        let header_size = DATAGRAM_HEADER_SIZE_FULL;
+        // Large
+        let header_size = DATAGRAM_HEADER_SIZE_LARGE;
 
-        if data.len() < header_size {
+        let data_len = ((data[1] as usize) << 8) |
+                       ((data[2] as usize)     );
+
+        let total_size = header_size + data_len;
+
+        if data.len() < total_size {
             return None;
         }
 
-        let channel_id = header_byte & 0x3F;
+        let channel_id = data[0] & 0x3F;
 
-        let sequence_id = ((data[1] as u32) << 24) |
-                          ((data[2] as u32) << 16) |
-                          ((data[3] as u32) <<  8) |
-                          ((data[4] as u32)      );
+        let sequence_id = (((data[3] & 0x0F) as u32) << 16) |
+                          (((data[4]       ) as u32) <<  8) |
+                          (((data[5]       ) as u32)      );
 
-        let window_parent_lead = ((data[5] as u16) << 8) |
-                                 ((data[6] as u16)     );
+        let window_parent_lead = ((data[6] as u16) << 8) |
+                                 ((data[7] as u16)     );
 
-        let channel_parent_lead = ((data[7] as u16) << 8) |
-                                  ((data[8] as u16)     );
+        let channel_parent_lead = ((data[8] as u16) << 8) |
+                                  ((data[9] as u16)     );
 
-        let data_len_u16 = ((data[ 9] as u16) << 8) |
-                           ((data[10] as u16)     );
-        let data_len = data_len_u16 as usize;
+        let fragment_id = ((data[10] as u16) << 8) |
+                          ((data[11] as u16)     );
 
-        if data.len() < header_size + data_len {
-            return None;
-        }
+        let fragment_id_last = ((data[12] as u16) << 8) |
+                               ((data[13] as u16)     );
 
         let data = data[header_size .. header_size + data_len].into();
 
@@ -196,9 +237,10 @@ fn read_datagram(data: &[u8]) -> Option<(Datagram, usize)> {
             sequence_id,
             window_parent_lead,
             channel_parent_lead,
-            fragment_id: FragmentId { id: 0, last: 0 },
+            fragment_id,
+            fragment_id_last,
             data,
-        }, header_size + data_len));
+        }, total_size));
     }
 }
 
@@ -214,18 +256,17 @@ fn read_data_payload(data: &[u8]) -> Option<Frame> {
                       ((data[2] as u32) <<  8) |
                       ((data[3] as u32)      );
 
-    let nonce = data[4] != 0;
+    let nonce = data[4] & 0x80 != 0x00;
 
-    let datagram_num = ((data[5] as u16) << 8) |
-                       ((data[6] as u16)     );
+    let datagram_num = data[4] & 0x7F;
 
-    let mut data_slice = &data[DATA_FRAME_PAYLOAD_HEADER_SIZE..];
+    let mut data_slice = &data[DATA_FRAME_PAYLOAD_HEADER_SIZE ..];
     let mut datagrams = Vec::new();
 
     for _ in 0 .. datagram_num {
         if let Some((datagram, read_size)) = read_datagram(data_slice) {
             datagrams.push(datagram);
-            data_slice = &data_slice[read_size..];
+            data_slice = &data_slice[read_size ..];
         } else {
             return None;
         }
@@ -309,13 +350,13 @@ fn read_ack_payload(data: &[u8]) -> Option<Frame> {
     let frame_ack_num = ((data[8] as u16) << 8) |
                         ((data[9] as u16)     );
 
-    let mut data_slice = &data[ACK_FRAME_PAYLOAD_HEADER_SIZE..];
+    let mut data_slice = &data[ACK_FRAME_PAYLOAD_HEADER_SIZE ..];
     let mut frame_acks = Vec::new();
 
     for _ in 0 .. frame_ack_num {
         if let Some((frame_ack, read_size)) = read_frame_ack(data_slice) {
             frame_acks.push(frame_ack);
-            data_slice = &data_slice[read_size..];
+            data_slice = &data_slice[read_size ..];
         } else {
             return None;
         }
@@ -330,14 +371,13 @@ fn read_ack_payload(data: &[u8]) -> Option<Frame> {
 
 
 fn write_connect(frame: &ConnectFrame) -> Box<[u8]> {
-    Box::new([
+    let mut frame_bytes = Box::new([
         CONNECT_FRAME_ID,
         frame.version,
         (frame.nonce >> 24) as u8,
         (frame.nonce >> 16) as u8,
         (frame.nonce >>  8) as u8,
         (frame.nonce      ) as u8,
-        frame.channel_count_sup,
         (frame.max_receive_rate >> 24) as u8,
         (frame.max_receive_rate >> 16) as u8,
         (frame.max_receive_rate >>  8) as u8,
@@ -350,29 +390,89 @@ fn write_connect(frame: &ConnectFrame) -> Box<[u8]> {
         (frame.max_receive_alloc >> 16) as u8,
         (frame.max_receive_alloc >>  8) as u8,
         (frame.max_receive_alloc      ) as u8,
-    ])
+        0,
+        0,
+        0,
+        0,
+    ]);
+
+    let frame_len = frame_bytes.len();
+    let data_bytes = &frame_bytes[0 .. frame_len - 4];
+
+    let crc = crc::compute(&data_bytes);
+    frame_bytes[frame_len - 4] = (crc >> 24) as u8;
+    frame_bytes[frame_len - 3] = (crc >> 16) as u8;
+    frame_bytes[frame_len - 2] = (crc >>  8) as u8;
+    frame_bytes[frame_len - 1] = (crc      ) as u8;
+
+    return frame_bytes;
 }
 
 fn write_connect_ack(frame: &ConnectAckFrame) -> Box<[u8]> {
-    Box::new([
+    let mut frame_bytes = Box::new([
         CONNECT_ACK_FRAME_ID,
         (frame.nonce >> 24) as u8,
         (frame.nonce >> 16) as u8,
         (frame.nonce >>  8) as u8,
         (frame.nonce      ) as u8,
-    ])
+        0,
+        0,
+        0,
+        0,
+    ]);
+
+    let frame_len = frame_bytes.len();
+    let data_bytes = &frame_bytes[0 .. frame_len - 4];
+
+    let crc = crc::compute(&data_bytes);
+    frame_bytes[frame_len - 4] = (crc >> 24) as u8;
+    frame_bytes[frame_len - 3] = (crc >> 16) as u8;
+    frame_bytes[frame_len - 2] = (crc >>  8) as u8;
+    frame_bytes[frame_len - 1] = (crc      ) as u8;
+
+    return frame_bytes;
 }
 
 fn write_disconnect(_frame: &DisconnectFrame) -> Box<[u8]> {
-    Box::new([
-        DISCONNECT_FRAME_ID
-    ])
+    let mut frame_bytes = Box::new([
+        DISCONNECT_FRAME_ID,
+        0,
+        0,
+        0,
+        0,
+    ]);
+
+    let frame_len = frame_bytes.len();
+    let data_bytes = &frame_bytes[0 .. frame_len - 4];
+
+    let crc = crc::compute(&data_bytes);
+    frame_bytes[frame_len - 4] = (crc >> 24) as u8;
+    frame_bytes[frame_len - 3] = (crc >> 16) as u8;
+    frame_bytes[frame_len - 2] = (crc >>  8) as u8;
+    frame_bytes[frame_len - 1] = (crc      ) as u8;
+
+    return frame_bytes;
 }
 
 fn write_disconnect_ack(_frame: &DisconnectAckFrame) -> Box<[u8]> {
-    Box::new([
-        DISCONNECT_ACK_FRAME_ID
-    ])
+    let mut frame_bytes = Box::new([
+        DISCONNECT_ACK_FRAME_ID,
+        0,
+        0,
+        0,
+        0,
+    ]);
+
+    let frame_len = frame_bytes.len();
+    let data_bytes = &frame_bytes[0 .. frame_len - 4];
+
+    let crc = crc::compute(&data_bytes);
+    frame_bytes[frame_len - 4] = (crc >> 24) as u8;
+    frame_bytes[frame_len - 3] = (crc >> 16) as u8;
+    frame_bytes[frame_len - 2] = (crc >>  8) as u8;
+    frame_bytes[frame_len - 1] = (crc      ) as u8;
+
+    return frame_bytes;
 }
 
 fn write_data(frame: &DataFrame) -> Box<[u8]> {
@@ -392,7 +492,7 @@ fn write_sync(frame: &SyncFrame) -> Box<[u8]> {
     let next_frame_id = frame.next_frame_id.unwrap_or(0);
     let next_packet_id = frame.next_packet_id.unwrap_or(0);
 
-    Box::new([
+    let mut frame_bytes = Box::new([
         SYNC_FRAME_ID,
         mode,
         (next_frame_id >> 24) as u8,
@@ -403,7 +503,22 @@ fn write_sync(frame: &SyncFrame) -> Box<[u8]> {
         (next_packet_id >> 16) as u8,
         (next_packet_id >>  8) as u8,
         (next_packet_id      ) as u8,
-    ])
+        0,
+        0,
+        0,
+        0,
+    ]);
+
+    let frame_len = frame_bytes.len();
+    let data_bytes = &frame_bytes[0 .. frame_len - 4];
+
+    let crc = crc::compute(&data_bytes);
+    frame_bytes[frame_len - 4] = (crc >> 24) as u8;
+    frame_bytes[frame_len - 3] = (crc >> 16) as u8;
+    frame_bytes[frame_len - 2] = (crc >>  8) as u8;
+    frame_bytes[frame_len - 1] = (crc      ) as u8;
+
+    return frame_bytes;
 }
 
 fn write_ack(frame: &AckFrame) -> Box<[u8]> {
@@ -422,19 +537,33 @@ pub trait Serialize {
 }
 
 impl Serialize for Frame {
-    fn read(data: &[u8]) -> Option<Self> {
-        if data.len() == 0 {
+    fn read(frame_bytes: &[u8]) -> Option<Self> {
+        if frame_bytes.len() < 5 {
             return None;
         }
 
-        match data[0] {
-            CONNECT_FRAME_ID => read_connect_payload(&data[1..]),
-            CONNECT_ACK_FRAME_ID => read_connect_ack_payload(&data[1..]),
-            DISCONNECT_FRAME_ID => read_disconnect_payload(&data[1..]),
-            DISCONNECT_ACK_FRAME_ID => read_disconnect_ack_payload(&data[1..]),
-            DATA_FRAME_ID => read_data_payload(&data[1..]),
-            SYNC_FRAME_ID => read_sync_payload(&data[1..]),
-            ACK_FRAME_ID => read_ack_payload(&data[1..]),
+        let frame_len = frame_bytes.len();
+        let data_bytes = &frame_bytes[0 .. frame_len - 4];
+
+        let crc = ((frame_bytes[frame_len - 4] as u32) << 24) |
+                  ((frame_bytes[frame_len - 3] as u32) << 16) |
+                  ((frame_bytes[frame_len - 2] as u32) <<  8) |
+                  ((frame_bytes[frame_len - 1] as u32)      );
+
+        if crc::compute(&data_bytes) != crc {
+            return None;
+        }
+
+        let payload_bytes = &frame_bytes[1 .. frame_len - 4];
+
+        match frame_bytes[0] {
+            CONNECT_FRAME_ID => read_connect_payload(payload_bytes),
+            CONNECT_ACK_FRAME_ID => read_connect_ack_payload(payload_bytes),
+            DISCONNECT_FRAME_ID => read_disconnect_payload(payload_bytes),
+            DISCONNECT_ACK_FRAME_ID => read_disconnect_ack_payload(payload_bytes),
+            DATA_FRAME_ID => read_data_payload(payload_bytes),
+            SYNC_FRAME_ID => read_sync_payload(payload_bytes),
+            ACK_FRAME_ID => read_ack_payload(payload_bytes),
             _ => None,
         }
     }
@@ -460,7 +589,7 @@ mod tests {
         println!("frame: {:#?}", f);
 
         let bytes = f.write();
-        println!("frame bytes: {:?}", bytes);
+        println!("frame bytes: {:02X?}", bytes);
 
         let f2 = Frame::read(&bytes).unwrap();
 
@@ -483,8 +612,8 @@ mod tests {
     fn verify_truncation_fails(f: &Frame) {
         let bytes = f.write();
 
-        for i in 1..bytes.len() {
-            let bytes_trunc = &bytes[0..i];
+        for i in 1 .. bytes.len() {
+            let bytes_trunc = &bytes[0 .. i];
             assert_eq!(Frame::read(&bytes_trunc), None);
         }
     }
@@ -494,8 +623,9 @@ mod tests {
         let f = Frame::ConnectFrame(ConnectFrame {
             version: 0x7F,
             nonce: 0x18273645,
-            channel_count_sup: 3,
+
             max_receive_rate: 0x98765432,
+
             max_packet_size: 0x01234567,
             max_receive_alloc: 0xABCDEF01,
         });
@@ -532,30 +662,37 @@ mod tests {
 
     #[test]
     fn data_basic() {
+        let small_data = (0 .. 256).map(|v| v as u8).collect::<Vec<_>>().into_boxed_slice();
+
         let f = Frame::DataFrame(DataFrame {
             sequence_id: 0x010203,
             nonce: true,
             datagrams: vec![
                 Datagram {
-                    sequence_id: 0x12345678,
+                    sequence_id: 0x45678,
                     channel_id: 63,
                     window_parent_lead: 0x34A8,
                     channel_parent_lead: 0x8A43,
-                    fragment_id: FragmentId {
-                        id: 0x4789,
-                        last: 0x478A,
-                    },
+                    fragment_id: 0x4789,
+                    fragment_id_last: 0x478A,
                     data: vec![ 0x00, 0x01, 0x02 ].into_boxed_slice(),
                 },
                 Datagram {
-                    sequence_id: 0x12345678,
+                    sequence_id: 0x12345,
                     channel_id: 63,
                     window_parent_lead: 0x34A8,
                     channel_parent_lead: 0x8A43,
-                    fragment_id: FragmentId {
-                        id: 0,
-                        last: 0,
-                    },
+                    fragment_id: 0,
+                    fragment_id_last: 0,
+                    data: small_data,
+                },
+                Datagram {
+                    sequence_id: 0x12345,
+                    channel_id: 63,
+                    window_parent_lead: 0x34A8,
+                    channel_parent_lead: 0x8A43,
+                    fragment_id: 0,
+                    fragment_id_last: 0,
                     data: vec![ 0x00, 0x01, 0x02 ].into_boxed_slice(),
                 },
             ],
@@ -629,12 +766,13 @@ mod tests {
     fn connect_random() {
         const NUM_ROUNDS: usize = 100;
 
-        for _ in 0..NUM_ROUNDS {
+        for _ in 0 .. NUM_ROUNDS {
             let f = Frame::ConnectFrame(ConnectFrame {
                 version: rand::random::<u8>(),
                 nonce: rand::random::<u32>(),
-                channel_count_sup: rand::random::<u8>(),
+
                 max_receive_rate: rand::random::<u32>(),
+
                 max_packet_size: rand::random::<u32>(),
                 max_receive_alloc: rand::random::<u32>(),
             });
@@ -648,7 +786,7 @@ mod tests {
     fn connect_ack_random() {
         const NUM_ROUNDS: usize = 100;
 
-        for _ in 0..NUM_ROUNDS {
+        for _ in 0 .. NUM_ROUNDS {
             let f = Frame::ConnectAckFrame(ConnectAckFrame {
                 nonce: rand::random::<u32>(),
             });
@@ -658,54 +796,79 @@ mod tests {
         }
     }
 
-    fn random_data(size: usize) -> Box<[u8]> {
-        (0..size).map(|_| rand::random::<u8>()).collect::<Vec<_>>().into_boxed_slice()
+    fn random_data(min: usize, max: usize) -> Box<[u8]> {
+        let len = rand::random::<usize>() % (max - min + 1);
+        (0 .. len).map(|_| rand::random::<u8>()).collect::<Vec<_>>().into_boxed_slice()
+    }
+
+    fn random_data_frame() -> Frame {
+        use crate::packet_id;
+
+        const MAX_DATAGRAMS: usize = 64;
+        const MAX_DATA_SIZE: usize = 100;
+
+        let mut datagrams = Vec::new();
+
+        for _ in 0 .. rand::random::<usize>() % MAX_DATAGRAMS {
+            let datagram = match rand::random::<u32>() % 3 {
+                0 => Datagram {
+                    // Micro
+                    sequence_id: rand::random::<u32>() & packet_id::MASK,
+                    channel_id: (rand::random::<usize>() % MAX_CHANNELS) as u8,
+                    window_parent_lead: rand::random::<u16>() % 128,
+                    channel_parent_lead: rand::random::<u16>() % 256,
+                    fragment_id: 0,
+                    fragment_id_last: 0,
+                    data: random_data(0, 64),
+                },
+                1 => Datagram {
+                    // Small
+                    sequence_id: rand::random::<u32>() & packet_id::MASK,
+                    channel_id: (rand::random::<usize>() % MAX_CHANNELS) as u8,
+                    window_parent_lead: rand::random::<u16>(),
+                    channel_parent_lead: rand::random::<u16>(),
+                    fragment_id: 0,
+                    fragment_id_last: 0,
+                    data: random_data(64, MAX_DATA_SIZE),
+                },
+                2 => {
+                    let mut fragment_id_last = rand::random::<u16>();
+                    let mut fragment_id = rand::random::<u16>();
+
+                    if fragment_id > fragment_id_last {
+                        std::mem::swap(&mut fragment_id, &mut fragment_id_last);
+                    }
+
+                    Datagram {
+                        // Large
+                        sequence_id: rand::random::<u32>() & packet_id::MASK,
+                        channel_id: (rand::random::<usize>() % MAX_CHANNELS) as u8,
+                        window_parent_lead: rand::random::<u16>(),
+                        channel_parent_lead: rand::random::<u16>(),
+                        fragment_id,
+                        fragment_id_last,
+                        data: random_data(0, MAX_DATA_SIZE),
+                    }
+                }
+                _ => panic!()
+            };
+
+            datagrams.push(datagram);
+        }
+
+        return Frame::DataFrame(DataFrame {
+            sequence_id: rand::random::<u32>(),
+            nonce: rand::random::<bool>(),
+            datagrams: datagrams,
+        });
     }
 
     #[test]
     fn data_random() {
         const NUM_ROUNDS: usize = 100;
-        const MAX_DATAGRAMS: usize = 100;
-        const MAX_DATA_SIZE: usize = 100;
 
-        for _ in 0..NUM_ROUNDS {
-            let mut datagrams = Vec::new();
-
-            for _ in 0 .. rand::random::<usize>() % MAX_DATAGRAMS {
-                let datagram = match rand::random::<u32>() % 2 {
-                    0 => Datagram {
-                        sequence_id: rand::random::<u32>(),
-                        channel_id: (rand::random::<usize>() % MAX_CHANNELS) as u8,
-                        window_parent_lead: rand::random::<u16>(),
-                        channel_parent_lead: rand::random::<u16>(),
-                        fragment_id: FragmentId {
-                            id: rand::random::<u16>(),
-                            last: rand::random::<u16>(),
-                        },
-                        data: random_data(MAX_DATA_SIZE),
-                    },
-                    1 => Datagram {
-                        sequence_id: rand::random::<u32>(),
-                        channel_id: (rand::random::<usize>() % MAX_CHANNELS) as u8,
-                        window_parent_lead: rand::random::<u16>(),
-                        channel_parent_lead: rand::random::<u16>(),
-                        fragment_id: FragmentId {
-                            id: 0,
-                            last: 0,
-                        },
-                        data: random_data(MAX_DATA_SIZE),
-                    },
-                    _ => panic!()
-                };
-
-                datagrams.push(datagram);
-            }
-
-            let f = Frame::DataFrame(DataFrame {
-                sequence_id: rand::random::<u32>(),
-                nonce: rand::random::<bool>(),
-                datagrams: datagrams,
-            });
+        for _ in 0 .. NUM_ROUNDS {
+            let f = random_data_frame();
 
             verify_consistent(&f);
             verify_extra_bytes_fail(&f);
@@ -717,7 +880,7 @@ mod tests {
     fn sync_random() {
         const NUM_ROUNDS: usize = 100;
 
-        for _ in 0..NUM_ROUNDS {
+        for _ in 0 .. NUM_ROUNDS {
             let f = Frame::SyncFrame(SyncFrame {
                 next_frame_id: if rand::random::<u32>() % 5 != 0 { Some(rand::random::<u32>()) } else { None },
                 next_packet_id: if rand::random::<u32>() % 5 != 0 { Some(rand::random::<u32>()) } else { None },
@@ -733,7 +896,7 @@ mod tests {
         const NUM_ROUNDS: usize = 100;
         const MAX_FRAME_ACKS: usize = 100;
 
-        for _ in 0..NUM_ROUNDS {
+        for _ in 0 .. NUM_ROUNDS {
             let mut frame_acks = Vec::new();
 
             for _ in 0 .. rand::random::<usize>() % MAX_FRAME_ACKS {
@@ -757,5 +920,33 @@ mod tests {
             verify_truncation_fails(&f);
         }
     }
+
+    // In practice this has yet to fail for any number of flips. May be rarer than 1 in 2^32, lol.
+    #[test]
+    fn crc_flips() {
+        const NUM_ROUNDS: usize = 10000;
+
+        // The alleged strength of the chosen CRC
+        const NUM_FLIPS: usize = 5;
+        const MAX_LEN: usize = 8192;
+
+        for _ in 0 .. NUM_ROUNDS {
+            let f = random_data_frame();
+            let mut frame_bytes = f.write();
+
+            assert!(frame_bytes.len() <= MAX_LEN);
+
+            for _ in 0 .. NUM_FLIPS {
+                let bit_index = rand::random::<usize>() % (frame_bytes.len() * 8);
+
+                let bit = bit_index % 8;
+                let byte_index = bit_index / 8;
+
+                frame_bytes[byte_index] ^= 0x01 << bit;
+            }
+
+            assert_eq!(Frame::read(&frame_bytes), None);
+        }
+    }
 }
 
