@@ -23,13 +23,17 @@ pub struct Server {
 }
 
 impl Server {
-    /// Opens a UDP socket and creates a corresponding [`Server`](Self) object. The UDP socket is
-    /// bound to the provided address, and configured to be non-blocking. Any errors resulting from
-    /// socket initialization are forwarded to the caller.
+    /// Opens a non-blocking UDP socket bound to the provided address, and creates a corresponding
+    /// [`Server`](Self) object.
     ///
-    /// The server will limit the number of concurrent, non-zombie connections to `max_peer_count`,
-    /// and will silently ignore connection requests which would exceed that limit. Valid incoming
+    /// The server will limit the number of active connections to `max_peer_count`, and will
+    /// silently ignore connection requests which would exceed that limit. Otherwise valid incoming
     /// connections will be initialized according to `peer_config`.
+    ///
+    /// # Error Handling
+    ///
+    /// Any errors resulting from socket initialization are forwarded to the caller. This function
+    /// will panic if the given endpoint configuration is not valid.
     pub fn bind<A: net::ToSocketAddrs>(addr: A,
                                        max_peer_count: usize,
                                        peer_config: endpoint::Config) -> Result<Self, std::io::Error> {
@@ -63,19 +67,24 @@ impl Server {
     }
 
     /// Returns a number of [`Peer`](peer::Peer) objects representing new connections since the
-    /// last call to [`incoming()`](Self::incoming). Any connections which have since entered the
-    /// zombie state (timed out) will not be returned.
+    /// last call to [`incoming()`](Self::incoming). Any connections which have since timed out
+    /// will not be returned.
     pub fn incoming(&mut self) -> impl Iterator<Item = peer::Peer> {
         std::mem::take(&mut self.incoming_peers).into_iter()
     }
 
-    /// Processes UDP frames received since the last call to [`step()`](Self::step), and
-    /// sends any pending outbound frames (acknowledgements, keep-alives, packet data, etc.).
+    /// Flushes pending outbound frames, reads as many UDP frames as possible from the internal
+    /// socket, and updates the states of active connections. Call
+    /// [`Peer::poll_events()`](peer::Peer::poll_events) after calling this function to retrieve
+    /// incoming packets and connection status updates for an individual peer.
     ///
-    /// Current, non-zombie [`Peer`](peer::Peer) objects will be updated as relevant data is
-    /// received. Call [`Peer::poll_events()`](peer::Peer::poll_events) after calling this method
-    /// to retrieve incoming packets and connection status updates for an individual peer.
+    /// *Note*: Internally, `uflow` uses the [leaky bucket
+    /// algorithm](https://en.wikipedia.org/wiki/Leaky_bucket) to control the rate at which UDP
+    /// frames are sent. To ensure that data is transferred smoothly, this function should be
+    /// called relatively frequently (a minimum of once per connection round-trip time).
     pub fn step(&mut self) {
+        self.flush();
+
         let mut frame_data_buf = [0; MAX_FRAME_SIZE];
 
         while let Ok((frame_size, address)) = self.socket.recv_from(&mut frame_data_buf) {
@@ -86,17 +95,19 @@ impl Server {
             }
         }
 
-        self.flush();
+        for (_, endpoint) in self.endpoints.iter_mut() {
+            endpoint.borrow_mut().step();
+        }
 
         self.endpoints.retain(|_, endpoint| !endpoint.borrow().is_zombie());
         self.incoming_peers.retain(|client| !client.is_zombie());
     }
 
-    /// Sends any pending outbound frames (acknowledgements, keep-alives, packet data, etc.).
+    /// Sends as many pending outbound frames (packet data, acknowledgements, keep-alives, etc.) as
+    /// possible for each peer.
     pub fn flush(&mut self) {
         for (&address, endpoint) in self.endpoints.iter_mut() {
             let ref mut data_sink = UdpFrameSink::new(&self.socket, address);
-
             endpoint.borrow_mut().flush(data_sink);
         }
     }
@@ -109,7 +120,6 @@ impl Server {
     fn handle_frame(&mut self, address: net::SocketAddr, frame: frame::Frame) {
         if let Some(endpoint) = self.endpoints.get_mut(&address) {
             let ref mut data_sink = UdpFrameSink::new(&self.socket, address);
-
             endpoint.borrow_mut().handle_frame(frame, data_sink);
         } else {
             if self.endpoints.len() < self.max_peer_count as usize {
