@@ -6,8 +6,7 @@ use std::convert::TryInto;
 const NUM_CHANNELS: usize = 4;
 
 fn server_thread() -> Vec<md5::Digest> {
-    let cfg = uflow::EndpointConfig::default()
-        .channel_count(NUM_CHANNELS);
+    let cfg = uflow::EndpointConfig::default();
 
     let mut server = uflow::Server::bind("127.0.0.1:8888", 1, cfg).unwrap();
     let mut peers = Vec::new();
@@ -66,8 +65,7 @@ fn server_thread() -> Vec<md5::Digest> {
 fn client_thread() -> Vec<md5::Digest> {
     let mut client = uflow::Client::bind_any_ipv4().unwrap();
 
-    let cfg = uflow::EndpointConfig::default()
-        .channel_count(NUM_CHANNELS);
+    let cfg = uflow::EndpointConfig::default();
 
     let mut server_peer = client.connect("127.0.0.1:8888", cfg).expect("Invalid address");
 
@@ -137,6 +135,8 @@ fn client_thread() -> Vec<md5::Digest> {
             }
         }
 
+        client.flush();
+
         std::thread::sleep(std::time::Duration::from_millis(15));
     }
 
